# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/000_Testing_export.ipynb.

# %% auto 0
__all__ = ['dummy']

# %% ../nbs/000_Testing_export.ipynb 1
from ._application.tester import Tester
from ._testing.apache_kafka_broker import ApacheKafkaBroker
from ._testing.in_memory_broker import InMemoryBroker
from ._testing.local_redpanda_broker import LocalRedpandaBroker
from fastkafka._testing.test_utils import (
    display_docs,
    mock_AIOKafkaProducer_send,
    nb_safe_seed,
    run_script_and_cancel,
    true_after,
)

__all__ = [
    "InMemoryBroker",
    "ApacheKafkaBroker",
    "LocalRedpandaBroker",
    "Tester",
    "nb_safe_seed",
    "true_after",
    "mock_AIOKafkaProducer_send",
    "run_script_and_cancel",
    "display_docs",
]

# %% ../nbs/000_Testing_export.ipynb 3
def dummy() -> None:
    pass


dummy.__module__ = "_dummy"



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/013_ProducerDecorator.ipynb.

# %% auto 0
__all__ = ['BaseSubmodel', 'ProduceReturnTypes', 'ProduceCallable', 'KafkaEvent', 'producer_decorator']

# %% ../../nbs/013_ProducerDecorator.ipynb 1
import functools
import json
from asyncio import iscoroutinefunction  # do not use the version from inspect
from collections import namedtuple
from dataclasses import dataclass
from typing import *

from aiokafka import AIOKafkaProducer
from pydantic import BaseModel

# %% ../../nbs/013_ProducerDecorator.ipynb 3
BaseSubmodel = TypeVar("BaseSubmodel", bound=BaseModel)
BaseSubmodel


@dataclass
class KafkaEvent(Generic[BaseSubmodel]):
    """
    A generic class for representing Kafka events. Based on BaseSubmodel, bound to pydantic.BaseModel

    Attributes:
        message (BaseSubmodel): The message contained in the Kafka event, can be of type pydantic.BaseModel.
        key (bytes, optional): The optional key used to identify the Kafka event.
    """

    message: BaseSubmodel
    key: Optional[bytes] = None


KafkaEvent.__module__ = "fastkafka"

# %% ../../nbs/013_ProducerDecorator.ipynb 5
ProduceReturnTypes = Union[BaseModel, KafkaEvent[BaseModel]]

ProduceCallable = Union[
    Callable[..., ProduceReturnTypes], Callable[..., Awaitable[ProduceReturnTypes]]
]

# %% ../../nbs/013_ProducerDecorator.ipynb 6
def _to_json_utf8(o: Any) -> bytes:
    """Converts to JSON and then encodes with UTF-8"""
    if hasattr(o, "json"):
        return o.json().encode("utf-8")  # type: ignore
    else:
        return json.dumps(o).encode("utf-8")

# %% ../../nbs/013_ProducerDecorator.ipynb 8
def _wrap_in_event(message: Union[BaseModel, KafkaEvent]) -> KafkaEvent:
    return message if type(message) == KafkaEvent else KafkaEvent(message)

# %% ../../nbs/013_ProducerDecorator.ipynb 11
def producer_decorator(
    producer_store: Dict[str, Any], func: ProduceCallable, topic: str
) -> ProduceCallable:
    """todo: write documentation"""

    @functools.wraps(func)
    async def _produce_async(
        *args: List[Any],
        producer_store: Dict[str, Any] = producer_store,
        f: Callable[..., Awaitable[ProduceReturnTypes]] = func,  # type: ignore
        **kwargs: Any
    ) -> ProduceReturnTypes:
        return_val = await f(*args, **kwargs)
        wrapped_val = _wrap_in_event(return_val)
        _, producer, _ = producer_store[topic]
        fut = await producer.send(
            topic, _to_json_utf8(wrapped_val.message), key=wrapped_val.key
        )
        msg = await fut
        return return_val

    @functools.wraps(func)
    def _produce_sync(
        *args: List[Any],
        producer_store: Dict[str, Any] = producer_store,
        f: Callable[..., ProduceReturnTypes] = func,  # type: ignore
        **kwargs: Any
    ) -> ProduceReturnTypes:
        return_val = f(*args, **kwargs)
        wrapped_val = _wrap_in_event(return_val)
        _, producer, _ = producer_store[topic]
        producer.send(topic, _to_json_utf8(wrapped_val.message), key=wrapped_val.key)
        return return_val

    return _produce_async if iscoroutinefunction(func) else _produce_sync



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/025_CLI_Testing.ipynb.

# %% auto 0
__all__ = ['logger', 'testing_install_deps']

# %% ../nbs/025_CLI_Testing.ipynb 1
from typing import *

import typer

from ._components.logger import get_logger
from ._components.test_dependencies import _install_testing_deps

# %% ../nbs/025_CLI_Testing.ipynb 5
logger = get_logger(__name__)

# %% ../nbs/025_CLI_Testing.ipynb 8
_testing_app = typer.Typer(help="Commands for managing fastkafka testing")

# %% ../nbs/025_CLI_Testing.ipynb 9
@_testing_app.command(
    "install_deps",
    help="Installs dependencies for FastKafka app testing",
)
def testing_install_deps() -> None:
    try:
        _install_testing_deps()
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        raise typer.Exit(1)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/004_Test_Utils.ipynb.

# %% auto 0
__all__ = ['logger', 'nb_safe_seed', 'true_after', 'mock_AIOKafkaProducer_send', 'run_script_and_cancel', 'display_docs']

# %% ../../nbs/004_Test_Utils.ipynb 1
import asyncio
import hashlib
import shlex
import subprocess  # nosec
import unittest
import unittest.mock
from contextlib import contextmanager
from datetime import datetime, timedelta
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import *

import asyncer
from aiokafka import AIOKafkaProducer
from IPython.display import IFrame

from .._application.app import FastKafka
from .._components._subprocess import terminate_asyncio_process
from .._components.helpers import _import_from_string, change_dir
from .._components.logger import get_logger

# %% ../../nbs/004_Test_Utils.ipynb 4
logger = get_logger(__name__)

# %% ../../nbs/004_Test_Utils.ipynb 6
def nb_safe_seed(s: str) -> Callable[[int], int]:
    """Gets a unique seed function for a notebook

    Params:
        s: name of the notebook used to initialize the seed function

    Returns:
        A unique seed function
    """
    init_seed = int(hashlib.sha256(s.encode("utf-8")).hexdigest(), 16) % (10**8)

    def _get_seed(x: int = 0, *, init_seed: int = init_seed) -> int:
        return init_seed + x

    return _get_seed

# %% ../../nbs/004_Test_Utils.ipynb 8
def true_after(seconds: float) -> Callable[[], bool]:
    """Function returning True after a given number of seconds"""
    t = datetime.now()

    def _true_after(seconds: float = seconds, t: datetime = t) -> bool:
        return (datetime.now() - t) > timedelta(seconds=seconds)

    return _true_after

# %% ../../nbs/004_Test_Utils.ipynb 10
@contextmanager
def mock_AIOKafkaProducer_send() -> Generator[unittest.mock.Mock, None, None]:
    """Mocks **send** method of **AIOKafkaProducer**"""
    with unittest.mock.patch("__main__.AIOKafkaProducer.send") as mock:

        async def _f() -> None:
            pass

        mock.return_value = asyncio.create_task(_f())

        yield mock

# %% ../../nbs/004_Test_Utils.ipynb 11
async def run_script_and_cancel(
    script: str,
    *,
    script_file: Optional[str] = None,
    cmd: Optional[str] = None,
    cancel_after: int = 10,
    app_name: str = "app",
    kafka_app_name: str = "kafka_app",
    generate_docs: bool = False,
) -> Tuple[int, bytes]:
    """Run script and cancel after predefined time

    Args:
        script: a python source code to be executed in a separate subprocess
        script_file: name of the script where script source will be saved
        cmd: command to execute. If None, it will be set to 'python3 -m {Path(script_file).stem}'
        cancel_after: number of seconds before sending SIGTERM signal

    Returns:
        A tuple containing exit code and combined stdout and stderr as a binary string
    """
    if script_file is None:
        script_file = "script.py"

    if cmd is None:
        cmd = f"python3 -m {Path(script_file).stem}"

    with TemporaryDirectory() as d:
        consumer_script = Path(d) / script_file

        with open(consumer_script, "w") as file:
            file.write(script)

        if generate_docs:
            logger.info(
                f"Generating docs for: {Path(script_file).stem}:{kafka_app_name}"
            )
            try:
                kafka_app: FastKafka = _import_from_string(
                    f"{Path(script_file).stem}:{kafka_app_name}"
                )
                await asyncer.asyncify(kafka_app.create_docs)()
            except Exception as e:
                logger.warning(
                    f"Generating docs failed for: {Path(script_file).stem}:{kafka_app_name}, ignoring it for now."
                )

        proc = subprocess.Popen(  # nosec: [B603:subprocess_without_shell_equals_true] subprocess call - check for execution of untrusted input.
            shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=d
        )
        await asyncio.sleep(cancel_after)
        proc.terminate()
        output, _ = proc.communicate()

        return (proc.returncode, output)

# %% ../../nbs/004_Test_Utils.ipynb 16
async def display_docs(docs_path: str, port: int = 4000) -> None:
    with change_dir(docs_path):
        process = await asyncio.create_subprocess_exec(
            "python3",
            "-m",
            "http.server",
            f"{port}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        try:
            from google.colab.output import eval_js

            proxy = eval_js(f"google.colab.kernel.proxyPort({port})")
            logger.info("Google colab detected! Proxy adjusted.")
        except:
            proxy = f"http://localhost:{port}"
        finally:
            await asyncio.sleep(2)
            display(IFrame(f"{proxy}", 1000, 700))  # type: ignore
            await asyncio.sleep(2)
            await terminate_asyncio_process(process)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/998_Internal_Helpers.ipynb.

# %% auto 0
__all__ = ['logger', 'in_notebook', 'change_dir', 'ImportFromStringError']

# %% ../../nbs/998_Internal_Helpers.ipynb 2
def in_notebook() -> bool:
    try:
        from IPython import get_ipython

        if "IPKernelApp" not in get_ipython().config:
            return False
    except ImportError:
        return False
    except AttributeError:
        return False
    return True

# %% ../../nbs/998_Internal_Helpers.ipynb 4
import contextlib
import importlib
import os
import sys
from functools import wraps
from inspect import signature
from pathlib import Path
from typing import *

import docstring_parser
import typer

from .logger import get_logger
from .meta import delegates

# %% ../../nbs/998_Internal_Helpers.ipynb 6
logger = get_logger(__name__)

# %% ../../nbs/998_Internal_Helpers.ipynb 8
@contextlib.contextmanager
def change_dir(d: str) -> Generator[None, None, None]:
    curdir = os.getcwd()
    os.chdir(d)
    try:
        yield
    finally:
        os.chdir(curdir)

# %% ../../nbs/998_Internal_Helpers.ipynb 10
class ImportFromStringError(Exception):
    pass


def _import_from_string(import_str: str) -> Any:
    """Imports library from string

    Note:
        copied from https://github.com/encode/uvicorn/blob/master/uvicorn/importer.py

    Args:
        import_str: input string in form 'main:app'

    """
    sys.path.append(".")

    if not isinstance(import_str, str):
        return import_str

    module_str, _, attrs_str = import_str.partition(":")
    if not module_str or not attrs_str:
        message = (
            'Import string "{import_str}" must be in format "<module>:<attribute>".'
        )
        typer.secho(f"{message}", err=True, fg=typer.colors.RED)
        raise ImportFromStringError(message.format(import_str=import_str))

    try:
        # nosemgrep: python.lang.security.audit.non-literal-import.non-literal-import
        module = importlib.import_module(module_str)
    except ImportError as exc:
        if exc.name != module_str:
            raise exc from None
        message = 'Could not import module "{module_str}".'
        raise ImportFromStringError(message.format(module_str=module_str))

    instance = module
    try:
        for attr_str in attrs_str.split("."):
            instance = getattr(instance, attr_str)
    except AttributeError:
        message = 'Attribute "{attrs_str}" not found in module "{module_str}".'
        raise ImportFromStringError(
            message.format(attrs_str=attrs_str, module_str=module_str)
        )

    return instance



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/096_Docusaurus_Helper.ipynb.

# %% auto 0
__all__ = ['fix_invalid_syntax_in_markdown', 'generate_markdown_docs']

# %% ../nbs/096_Docusaurus_Helper.ipynb 2
import itertools
import re
import types
from inspect import Signature, getmembers, isclass, isfunction, signature
from pathlib import Path
from typing import *

from docstring_parser import parse
from docstring_parser.common import DocstringParam, DocstringRaises, DocstringReturns
from nbdev.config import get_config
from nbdev_mkdocs.mkdocs import (
    _add_all_submodules,
    _get_api_summary,
    _import_all_members,
    _import_functions_and_classes,
    _import_submodules,
)

# %% ../nbs/096_Docusaurus_Helper.ipynb 4
def _format_docstring_sections(
    items: Union[List[DocstringParam], List[DocstringReturns], List[DocstringRaises]],
    keyword: str,
) -> str:
    """Format a list of docstring sections

    Args:
        items: A list of DocstringParam objects
        keyword: The type of section to format (e.g. 'Parameters', 'Returns', 'Exceptions')

    Returns:
        The formatted docstring.
    """
    formatted_docstring = ""
    if len(items) > 0:
        formatted_docstring += f"**{keyword}**:\n"
        for item in items:
            if keyword == "Parameters":
                formatted_docstring += f"- `{item.arg_name}`: {item.description}\n"  # type: ignore
            elif keyword == "Exceptions":
                formatted_docstring += f"- `{item.type_name}`: {item.description}\n"
            else:
                formatted_docstring += f"- {item.description}\n"
        formatted_docstring = f"{formatted_docstring}\n"
    return formatted_docstring

# %% ../nbs/096_Docusaurus_Helper.ipynb 8
def _docstring_to_markdown(docstring: str) -> str:
    """Converts a docstring to a markdown-formatted string.

    Args:
        docstring: The docstring to convert.

    Returns:
        The markdown-formatted docstring.
    """
    parsed_docstring = parse(docstring)
    formatted_docstring = f"{parsed_docstring.short_description}\n\n"
    formatted_docstring += (
        f"{parsed_docstring.long_description}\n\n"
        if parsed_docstring.long_description
        else ""
    )
    formatted_docstring += _format_docstring_sections(
        parsed_docstring.params, "Parameters"
    )
    formatted_docstring += _format_docstring_sections(
        parsed_docstring.many_returns, "Returns"
    )
    formatted_docstring += _format_docstring_sections(
        parsed_docstring.raises, "Exceptions"
    )

    return formatted_docstring

# %% ../nbs/096_Docusaurus_Helper.ipynb 12
def _get_submodules(module_name: str) -> List[str]:
    """Get a list of all submodules contained within the module.

    Args:
        module_name: The name of the module to retrieve submodules from

    Returns:
        A list of submodule names within the module
    """
    members = _import_all_members(module_name)
    members_with_submodules = _add_all_submodules(members)
    members_with_submodules_str: List[str] = [
        x[:-1] if x.endswith(".") else x for x in members_with_submodules
    ]
    return members_with_submodules_str

# %% ../nbs/096_Docusaurus_Helper.ipynb 14
def _load_submodules(
    module_name: str, members_with_submodules: List[str]
) -> List[Union[types.FunctionType, Type[Any]]]:
    """Load the given submodules from the module.

    Args:
        module_name: The name of the module whose submodules to load
        members_with_submodules: A list of submodule names to load

    Returns:
        A list of imported submodule objects.
    """
    submodules = _import_submodules(module_name)
    members: List[Tuple[str, Union[types.FunctionType, Type[Any]]]] = list(
        itertools.chain(*[_import_functions_and_classes(m) for m in submodules])
    )
    names = [
        y
        for x, y in members
        if f"{y.__module__}.{y.__name__}" in members_with_submodules
    ]
    return names

# %% ../nbs/096_Docusaurus_Helper.ipynb 16
def _convert_union_to_optional(annotation_str: str) -> str:
    """Convert the 'Union[Type1, Type2, ..., NoneType]' to 'Optional[Type1, Type2, ...]' in the given annotation string

    Args:
        annotation_str: The type annotation string to convert.

    Returns:
        The converted type annotation string.
    """
    pattern = r"Union\[(.*)?,\s*NoneType\s*\]"
    match = re.search(pattern, annotation_str)
    if match:
        union_type = match.group(1)
        optional_type = f"Optional[{union_type}]"
        return re.sub(pattern, optional_type, annotation_str)
    else:
        return annotation_str

# %% ../nbs/096_Docusaurus_Helper.ipynb 18
def _get_arg_list_with_signature(_signature: Signature) -> str:
    """Converts a function's signature into a string representation of its argument list.

    Args:
        _signature (signature): The signature object for the function to convert.

    Returns:
        str: A string representation of the function's argument list.
    """
    arg_list = []
    for param in _signature.parameters.values():
        arg_list.append(_convert_union_to_optional(str(param)))

    return ", ".join(arg_list)

# %% ../nbs/096_Docusaurus_Helper.ipynb 21
def _get_symbol_definition(symbol: Union[types.FunctionType, Type[Any]]) -> str:
    """Return the definition of a given symbol.

    Args:
        symbol: A function or method object to get the definition for.

    Returns:
        A string representing the function definition
    """
    _signature = signature(symbol)
    arg_list = _get_arg_list_with_signature(_signature)
    ret_val = ""

    if isfunction(symbol):
        ret_val = f"`def {symbol.__name__}({arg_list})"
        if _signature.return_annotation and "inspect._empty" not in str(
            _signature.return_annotation
        ):
            if isinstance(_signature.return_annotation, type):
                ret_val = ret_val + f" -> {_signature.return_annotation.__name__}`\n"
            else:
                ret_val = ret_val + f" -> {_signature.return_annotation}`\n"

        else:
            ret_val = ret_val + " -> None`\n"

    return ret_val

# %% ../nbs/096_Docusaurus_Helper.ipynb 27
def _get_formatted_docstring_for_symbol(
    symbol: Union[types.FunctionType, Type[Any]]
) -> str:
    """Recursively parses and get formatted docstring of a symbol.

    Args:
        symbol: A Python class or function object to parse the docstring for.

    Returns:
        A formatted docstring of the symbol and its members.

    """

    def traverse(symbol: Union[types.FunctionType, Type[Any]], contents: str) -> str:
        """Recursively traverse the members of a symbol and append their docstrings to the provided contents string.

        Args:
            symbol: A Python class or function object to parse the docstring for.
            contents: The current formatted docstrings.

        Returns:
            The updated formatted docstrings.

        """
        for x, y in getmembers(symbol):
            if not x.startswith("_") or x.endswith("__"):
                if isfunction(y) and y.__doc__ is not None:
                    contents += f"{_get_symbol_definition(y)}\n{_docstring_to_markdown(y.__doc__)}"
                elif isclass(y) and not x.startswith("__") and y.__doc__ is not None:
                    contents += f"{_get_symbol_definition(y)}\n{_docstring_to_markdown(y.__doc__)}"
                    contents = traverse(y, contents)
        return contents

    contents = (
        f"{_get_symbol_definition(symbol)}\n{_docstring_to_markdown(symbol.__doc__)}"
        if symbol.__doc__ is not None
        else ""
    )
    if isclass(symbol):
        contents = traverse(symbol, contents)
    return contents

# %% ../nbs/096_Docusaurus_Helper.ipynb 31
def _convert_html_style_attribute_to_jsx(contents: str) -> str:
    """Converts the inline style attributes in an HTML string to JSX compatible format.

    Args:
        contents: A string containing an HTML document or fragment.

    Returns:
        A string with inline style attributes converted to JSX compatible format.
    """
    style_regex = re.compile(r'style="(.+?)"')
    style_matches = style_regex.findall(contents)

    for style_match in style_matches:
        style_dict = {}
        styles = style_match.split(";")
        for style in styles:
            key_value = style.split(":")
            if len(key_value) == 2:
                key = re.sub(
                    r"-(.)", lambda m: m.group(1).upper(), key_value[0].strip()
                )
                value = key_value[1].strip().replace("'", '"')
                style_dict[key] = value
        replacement = "style={{"
        for key, value in style_dict.items():
            replacement += f"{key}: '{value}', "
        replacement = replacement[:-2] + "}}"
        contents = contents.replace(f'style="{style_match}"', replacement)

    return contents

# %% ../nbs/096_Docusaurus_Helper.ipynb 33
def _get_all_markdown_files_path(docs_path: Path) -> List[Path]:
    """Get all Markdown files in a directory and its subdirectories.

    Args:
        directory: The path to the directory to search in.

    Returns:
        A list of paths to all Markdown files found in the directory and its subdirectories.
    """
    markdown_files = [file_path for file_path in docs_path.glob("**/*.md")]
    return markdown_files

# %% ../nbs/096_Docusaurus_Helper.ipynb 35
def _fix_special_symbols_in_html(contents: str) -> str:
    contents = contents.replace("”", '"')
    return contents

# %% ../nbs/096_Docusaurus_Helper.ipynb 37
def fix_invalid_syntax_in_markdown(docs_path: str) -> None:
    """Fix invalid HTML syntax in markdown files and converts inline style attributes to JSX-compatible format.

    Args:
        docs_path: The path to the root directory to search for markdown files.
    """
    markdown_files = _get_all_markdown_files_path(Path(docs_path))
    updated_contents = [
        _convert_html_style_attribute_to_jsx(Path(file).read_text())
        for file in markdown_files
    ]
    updated_contents = [
        _fix_special_symbols_in_html(contents) for contents in updated_contents
    ]
    for i, file_path in enumerate(markdown_files):
        file_path.write_text(updated_contents[i])

# %% ../nbs/096_Docusaurus_Helper.ipynb 39
def generate_markdown_docs(module_name: str, docs_path: str) -> None:
    """Generates Markdown documentation files for the symbols in the given module and save them to the given directory.

    Args:
        module_name: The name of the module to generate documentation for.
        docs_path: The path to the directory where the documentation files will be saved.
    """
    members_with_submodules = _get_submodules(module_name)
    symbols = _load_submodules(module_name, members_with_submodules)

    for symbol in symbols:
        content = f"`{symbol.__module__}.{symbol.__name__}`\n\n"
        content += _get_formatted_docstring_for_symbol(symbol)
        target_file_path = (
            "/".join(f"{symbol.__module__}.{symbol.__name__}".split(".")) + ".md"
        )

        with open((Path(docs_path) / "api" / target_file_path), "w") as f:
            f.write(content)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/023_CLI.ipynb.

# %% auto 0
__all__ = ['logger', 'run']

# %% ../nbs/023_CLI.ipynb 1
import asyncio
import multiprocessing
from typing import *

import typer

from . import _cli_docs, _cli_testing
from ._components.logger import get_logger
from ._server import run_fastkafka_server

# %% ../nbs/023_CLI.ipynb 5
logger = get_logger(__name__)

# %% ../nbs/023_CLI.ipynb 8
_app = typer.Typer(help="")

# %% ../nbs/023_CLI.ipynb 9
@_app.command(
    help="Runs Fast Kafka API application",
)
def run(
    num_workers: int = typer.Option(
        multiprocessing.cpu_count(),
        help="Number of FastKafka instances to run, defaults to number of CPU cores.",
    ),
    app: str = typer.Argument(
        ...,
        help="input in the form of 'path:app', where **path** is the path to a python file and **app** is an object of type **FastKafka**.",
    ),
    kafka_broker: str = typer.Option(
        ...,
        help="kafka_broker, one of the keys of the kafka_brokers dictionary passed in the constructor of FastaKafka class.",
    ),
) -> None:
    try:
        asyncio.run(
            run_fastkafka_server(
                num_workers=num_workers, app=app, kafka_broker=kafka_broker
            )
        )
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        raise typer.Exit(1)

# %% ../nbs/023_CLI.ipynb 13
_app.add_typer(_cli_docs._docs_app, name="docs")

# %% ../nbs/023_CLI.ipynb 21
_app.add_typer(_cli_testing._testing_app, name="testing")



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/016_Tester.ipynb.

# %% auto 0
__all__ = ['Tester', 'mirror_producer', 'mirror_consumer']

# %% ../../nbs/016_Tester.ipynb 1
import asyncio
import inspect
from contextlib import asynccontextmanager
from typing import *

from pydantic import BaseModel

from .app import FastKafka
from .._components.meta import delegates, patch
from .._testing.apache_kafka_broker import ApacheKafkaBroker
from .._testing.in_memory_broker import InMemoryBroker
from .._testing.local_redpanda_broker import LocalRedpandaBroker

# %% ../../nbs/016_Tester.ipynb 6
class Tester(FastKafka):
    @delegates(ApacheKafkaBroker.__init__)
    def __init__(
        self,
        app: Union[FastKafka, List[FastKafka]],
        *,
        broker: Optional[
            Union[ApacheKafkaBroker, LocalRedpandaBroker, InMemoryBroker]
        ] = None,
    ):
        """Mirror-like object for testing a FastFafka application

        Can be used as context manager

        """
        self.apps = app if isinstance(app, list) else [app]
        host, port = self.apps[0]._kafka_config["bootstrap_servers"].split(":")
        super().__init__(kafka_brokers={"localhost": {"url": host, "port": port}})
        self.create_mirrors()

        self.broker = broker

    @delegates(LocalRedpandaBroker.__init__)
    def using_local_redpanda(self, **kwargs: Any) -> "Tester":
        """Starts local Redpanda broker used by the Tester instance

        Args:
            listener_port: Port on which the clients (producers and consumers) can connect
            tag: Tag of Redpanda image to use to start container
            seastar_core: Core(s) to use byt Seastar (the framework Redpanda uses under the hood)
            memory: The amount of memory to make available to Redpanda
            mode: Mode to use to load configuration properties in container
            default_log_level: Log levels to use for Redpanda
            topics: List of topics to create after sucessfull redpanda broker startup
            retries: Number of retries to create redpanda service
            apply_nest_asyncio: set to True if running in notebook
            port allocation if the requested port was taken

        Returns:
            An instance of tester with Redpanda as broker
        """
        topics = set().union(*(app.get_topics() for app in self.apps))
        kwargs["topics"] = (
            topics.union(kwargs["topics"]) if "topics" in kwargs else topics
        )
        self.broker = LocalRedpandaBroker(**kwargs)

        return self

    @delegates(ApacheKafkaBroker.__init__)
    def using_local_kafka(self, **kwargs: Any) -> "Tester":
        """Starts local Kafka broker used by the Tester instance

        Args:
            data_dir: Path to the directory where the zookeepeer instance will save data
            zookeeper_port: Port for clients (Kafka brokes) to connect
            listener_port: Port on which the clients (producers and consumers) can connect
            topics: List of topics to create after sucessfull Kafka broker startup
            retries: Number of retries to create kafka and zookeeper services using random
            apply_nest_asyncio: set to True if running in notebook
            port allocation if the requested port was taken

        Returns:
            An instance of tester with Kafka as broker
        """
        topics = set().union(*(app.get_topics() for app in self.apps))
        kwargs["topics"] = (
            topics.union(kwargs["topics"]) if "topics" in kwargs else topics
        )
        self.broker = ApacheKafkaBroker(**kwargs)

        return self

    async def _start_tester(self) -> None:
        """Starts the Tester"""
        for app in self.apps:
            app.create_mocks()
            await app.__aenter__()
        self.create_mocks()
        await super().__aenter__()
        await asyncio.sleep(3)

    async def _stop_tester(self) -> None:
        """Shuts down the Tester"""
        await super().__aexit__(None, None, None)
        for app in self.apps[::-1]:
            await app.__aexit__(None, None, None)

    def create_mirrors(self) -> None:
        pass

    @asynccontextmanager
    async def _create_ctx(self) -> AsyncGenerator["Tester", None]:
        if self.broker is None:
            topics = set().union(*(app.get_topics() for app in self.apps))
            self.broker = InMemoryBroker()

        bootstrap_server = await self.broker._start()
        old_bootstrap_servers: List[str] = list()
        try:
            if isinstance(self.broker, (ApacheKafkaBroker, LocalRedpandaBroker)):
                self._set_bootstrap_servers(bootstrap_servers=bootstrap_server)
                for app in self.apps:
                    old_bootstrap_servers.append(app._kafka_config["bootstrap_servers"])
                    app._set_bootstrap_servers(bootstrap_server)
            await self._start_tester()
            try:
                yield self
            finally:
                await self._stop_tester()
        finally:
            await self.broker._stop()
            for app, server in zip(self.apps, old_bootstrap_servers):
                app._set_bootstrap_servers(server)

    async def __aenter__(self) -> "Tester":
        self._ctx = self._create_ctx()
        return await self._ctx.__aenter__()

    async def __aexit__(self, *args: Any) -> None:
        await self._ctx.__aexit__(*args)


Tester.__module__ = "fastkafka.testing"

# %% ../../nbs/016_Tester.ipynb 11
def mirror_producer(topic: str, producer_f: Callable[..., Any]) -> Callable[..., Any]:
    msg_type = inspect.signature(producer_f).return_annotation

    async def skeleton_func(msg: BaseModel) -> None:
        pass

    mirror_func = skeleton_func
    sig = inspect.signature(skeleton_func)

    # adjust name
    mirror_func.__name__ = "on_" + topic

    # adjust arg and return val
    sig = sig.replace(
        parameters=[
            inspect.Parameter(
                name="msg",
                annotation=msg_type,
                kind=inspect.Parameter.POSITIONAL_OR_KEYWORD,
            )
        ]
    )

    mirror_func.__signature__ = sig  # type: ignore

    return mirror_func

# %% ../../nbs/016_Tester.ipynb 13
def mirror_consumer(topic: str, consumer_f: Callable[..., Any]) -> Callable[..., Any]:
    msg_type = inspect.signature(consumer_f).parameters["msg"]

    async def skeleton_func(msg: BaseModel) -> BaseModel:
        return msg

    mirror_func = skeleton_func
    sig = inspect.signature(skeleton_func)

    # adjust name
    mirror_func.__name__ = "to_" + topic

    # adjust arg and return val
    sig = sig.replace(parameters=[msg_type], return_annotation=msg_type.annotation)

    mirror_func.__signature__ = sig  # type: ignore
    return mirror_func

# %% ../../nbs/016_Tester.ipynb 15
@patch
def create_mirrors(self: Tester) -> None:
    for app in self.apps:
        for topic, (consumer_f, _) in app._consumers_store.items():
            mirror_f = mirror_consumer(topic, consumer_f)
            mirror_f = self.produces()(mirror_f)  # type: ignore
            setattr(self, mirror_f.__name__, mirror_f)
        for topic, (producer_f, _, _) in app._producers_store.items():
            mirror_f = mirror_producer(topic, producer_f)
            mirror_f = self.consumes()(mirror_f)  # type: ignore
            setattr(self, mirror_f.__name__, mirror_f)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/022_Subprocess.ipynb.

# %% auto 0
__all__ = ['logger', 'terminate_asyncio_process', 'run_async_subprocesses']

# %% ../../nbs/022_Subprocess.ipynb 1
import asyncio
import signal
from typing import *

import asyncer
import typer

from .logger import get_logger

# %% ../../nbs/022_Subprocess.ipynb 5
logger = get_logger(__name__)

# %% ../../nbs/022_Subprocess.ipynb 7
async def terminate_asyncio_process(p: asyncio.subprocess.Process) -> None:
    logger.info(f"terminate_asyncio_process(): Terminating the process {p.pid}...")
    # Check if SIGINT already propagated to process
    try:
        await asyncio.wait_for(p.wait(), 1)
        logger.info(
            f"terminate_asyncio_process(): Process {p.pid} was already terminated."
        )
        return
    except asyncio.TimeoutError:
        pass

    for i in range(3):
        p.terminate()
        try:
            await asyncio.wait_for(p.wait(), 10)
            logger.info(f"terminate_asyncio_process(): Process {p.pid} terminated.")
            return
        except asyncio.TimeoutError:
            logger.warning(
                f"terminate_asyncio_process(): Process {p.pid} not terminated, retrying..."
            )

    logger.warning(f"Killing the process {p.pid}...")
    p.kill()
    await p.wait()
    logger.warning(f"terminate_asyncio_process(): Process {p.pid} killed!")

# %% ../../nbs/022_Subprocess.ipynb 9
async def run_async_subprocesses(
    commands: List[str], commands_args: List[List[Any]], *, sleep_between: int = 0
) -> None:
    loop = asyncio.get_event_loop()

    HANDLED_SIGNALS = (
        signal.SIGINT,  # Unix signal 2. Sent by Ctrl+C.
        signal.SIGTERM,  # Unix signal 15. Sent by `kill <pid>`.
    )

    d = {"should_exit": False}

    def handle_exit(sig: int, d: Dict[str, bool] = d) -> None:
        d["should_exit"] = True

    for sig in HANDLED_SIGNALS:
        loop.add_signal_handler(sig, handle_exit, sig)

    async with asyncer.create_task_group() as tg:
        tasks = []
        for cmd, args in zip(commands, commands_args):
            tasks.append(
                tg.soonify(asyncio.create_subprocess_exec)(
                    cmd,
                    *args,
                    stdout=asyncio.subprocess.PIPE,
                    stdin=asyncio.subprocess.PIPE,
                )
            )
            await asyncio.sleep(sleep_between)

    procs = [task.value for task in tasks]

    async def log_output(
        output: Optional[asyncio.StreamReader], pid: int, d: Dict[str, bool] = d
    ) -> None:
        if output is None:
            raise RuntimeError("Expected StreamReader, got None. Is stdout piped?")
        while not output.at_eof():
            outs = await output.readline()
            if outs != b"":
                typer.echo(f"[{pid:03d}]: " + outs.decode("utf-8"), nl=False)

    async with asyncer.create_task_group() as tg:
        for proc in procs:
            tg.soonify(log_output)(proc.stdout, proc.pid)

        while not d["should_exit"]:
            await asyncio.sleep(0.2)

        typer.echo("Starting process cleanup, this may take a few seconds...")
        for proc in procs:
            tg.soonify(terminate_asyncio_process)(proc)

    for proc in procs:
        output, _ = await proc.communicate()
        if output:
            typer.echo(f"[{proc.pid:03d}]: " + output.decode("utf-8"), nl=False)

    returncodes = [proc.returncode for proc in procs]
    if not returncodes == [0] * len(procs):
        typer.secho(
            f"Return codes are not all zero: {returncodes}",
            err=True,
            fg=typer.colors.RED,
        )
        raise typer.Exit(1)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/024_CLI_Docs.ipynb.

# %% auto 0
__all__ = ['logger', 'docs_install_deps', 'generate_docs', 'serve_docs']

# %% ../nbs/024_CLI_Docs.ipynb 1
import asyncio
import signal
import socketserver
from http.server import SimpleHTTPRequestHandler
from types import FrameType
from typing import *

import typer

from fastkafka._components.docs_dependencies import (
    _check_npm_with_local,
    _install_docs_npm_deps,
    _install_node,
)
from ._components.helpers import _import_from_string, change_dir
from ._components.logger import get_logger

# %% ../nbs/024_CLI_Docs.ipynb 5
logger = get_logger(__name__)

# %% ../nbs/024_CLI_Docs.ipynb 8
_docs_app = typer.Typer(help="Commands for managing fastkafka app documentation")

# %% ../nbs/024_CLI_Docs.ipynb 9
@_docs_app.command(
    "install_deps",
    help="Installs dependencies for FastKafka documentation generation",
)
def docs_install_deps() -> None:
    try:
        _check_npm_with_local()
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        install_confirm = typer.confirm(
            "npm not found or version is too low, do you want us to install the NodeJS and npm locally?"
        )
        if install_confirm is False:
            print("Not installing NodeJS and npm locally, exiting..")
            raise typer.Abort()
        else:
            _install_node()
    asyncio.run(_install_docs_npm_deps())


@_docs_app.command(
    "generate",
    help="Generates documentation for a FastKafka application",
)
def generate_docs(
    root_path: str = typer.Option(
        ".", help="root path under which documentation will be created"
    ),
    app: str = typer.Argument(
        ...,
        help="input in the form of 'path:app', where **path** is the path to a python file and **app** is an object of type **FastKafka**.",
    ),
) -> None:
    try:
        application = _import_from_string(app)
        application.skip_docs = False
        application.create_docs()
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        raise typer.Exit(1)


@_docs_app.command(
    "serve",
    help="Generates and serves documentation for a FastKafka application",
)
def serve_docs(
    root_path: str = typer.Option(
        ".", help="root path under which documentation will be created"
    ),
    bind: str = typer.Option("127.0.0.1", help="Some info"),
    port: int = typer.Option(8000, help="Some info"),
    app: str = typer.Argument(
        ...,
        help="input in the form of 'path:app', where **path** is the path to a python file and **app** is an object of type **FastKafka**.",
    ),
) -> None:
    try:
        application = _import_from_string(app)
        application.create_docs()
        with change_dir("asyncapi/docs/"):
            server_address = (bind, port)
            handler = SimpleHTTPRequestHandler

            d = {"should_stop": False}

            def sigint_handler(
                signal: int, frame: Optional[FrameType], d: Dict[str, bool] = d
            ) -> None:
                d["should_stop"] = True

            signal.signal(signal.SIGINT, sigint_handler)
            signal.signal(signal.SIGTERM, sigint_handler)

            with socketserver.TCPServer(server_address, handler) as httpd:
                httpd.timeout = 0.1
                typer.secho(
                    f"Serving documentation on http://{server_address[0]}:{server_address[1]}"
                )
                while not d["should_stop"]:
                    httpd.handle_request()
                typer.secho(f"Interupting serving of documentation and cleaning up...")
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        raise typer.Exit(1)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/098_Test_Dependencies.ipynb.

# %% auto 0
__all__ = ['logger', 'kafka_version', 'kafka_fname', 'kafka_url', 'local_path', 'tgz_path', 'kafka_path', 'check_java',
           'check_kafka', 'generate_app_src', 'generate_app_in_tmp']

# %% ../../nbs/098_Test_Dependencies.ipynb 2
import shutil
import tarfile
from contextlib import contextmanager
from os import environ
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import *

from .helpers import change_dir, in_notebook
from .logger import get_logger

if in_notebook():
    from tqdm.notebook import tqdm
else:
    from tqdm import tqdm

# %% ../../nbs/098_Test_Dependencies.ipynb 4
logger = get_logger(__name__)

# %% ../../nbs/098_Test_Dependencies.ipynb 6
def check_java(*, potential_jdk_path: Optional[List[Path]] = None) -> bool:
    if potential_jdk_path is None:
        potential_jdk_path = list(Path(environ["HOME"] + "/.jdk").glob("jdk-11*"))

    if potential_jdk_path != []:
        logger.info("Java is already installed.")
        if not shutil.which("java"):
            logger.info("But not exported to PATH, exporting...")
            environ["PATH"] = environ["PATH"] + f":{potential_jdk_path[0]}/bin"
        return True
    return False

# %% ../../nbs/098_Test_Dependencies.ipynb 8
def _install_java() -> None:
    """Checks if jdk-11 is installed on the machine and installs it if not
    Returns:
       None
    """
    try:
        import jdk
    except Exception as e:
        msg = "Please install test version of fastkafka using 'pip install fastkafka[test]' command"
        logger.error(msg)
        raise RuntimeError(msg)

    if not check_java():
        logger.info("Installing Java...")
        logger.info(" - installing jdk...")
        jdk_bin_path = jdk.install("11")
        logger.info(f" - jdk path: {jdk_bin_path}")
        environ["PATH"] = environ["PATH"] + f":{jdk_bin_path}/bin"
        logger.info("Java installed.")

# %% ../../nbs/098_Test_Dependencies.ipynb 10
# ToDo: move it somewhere
kafka_version = "3.3.2"
kafka_fname = f"kafka_2.13-{kafka_version}"
kafka_url = f"https://dlcdn.apache.org/kafka/{kafka_version}/{kafka_fname}.tgz"
local_path = Path(environ["HOME"]) / ".local"
tgz_path = local_path / f"{kafka_fname}.tgz"
kafka_path = local_path / f"{kafka_fname}"


def check_kafka(kafka_path: Path = kafka_path) -> bool:
    if (kafka_path / "bin").exists():
        logger.info("Kafka is installed.")
        if not shutil.which("kafka-server-start.sh"):
            logger.info("But not exported to PATH, exporting...")
            environ["PATH"] = environ["PATH"] + f":{kafka_path}/bin"
        return True
    return False

# %% ../../nbs/098_Test_Dependencies.ipynb 11
def _install_kafka(
    *,
    kafka_url: str = kafka_url,
    local_path: Path = local_path,
    tgz_path: Path = tgz_path,
) -> None:
    """Checks if kafka is installed on the machine and installs it if not
    Returns:
       None
    """
    try:
        import requests
    except Exception as e:
        msg = "Please install test version of fastkafka using 'pip install fastkafka[test]' command"
        logger.error(msg)
        raise RuntimeError(msg)

    if not check_kafka():
        logger.info("Installing Kafka...")
        local_path.mkdir(exist_ok=True, parents=True)
        response = requests.get(
            kafka_url,
            stream=True,
        )
        try:
            total = response.raw.length_remaining // 128
        except Exception:
            total = None

        with open(tgz_path, "wb") as f:
            for data in tqdm(response.iter_content(chunk_size=128), total=total):
                f.write(data)

        with tarfile.open(tgz_path) as tar:
            for tarinfo in tar:
                tar.extract(tarinfo, local_path)

        environ["PATH"] = environ["PATH"] + f":{kafka_path}/bin"
        logger.info(f"Kafka installed in {kafka_path}.")

# %% ../../nbs/098_Test_Dependencies.ipynb 13
def _install_testing_deps() -> None:
    _install_java()
    _install_kafka()

# %% ../../nbs/098_Test_Dependencies.ipynb 15
def generate_app_src(out_path: Union[Path, str]) -> None:
    import nbformat
    from nbconvert import PythonExporter

    path = Path("099_Test_Service.ipynb")
    if not path.exists():
        path = Path("..") / "099_Test_Service.ipynb"
    if not path.exists():
        raise ValueError(f"Path '{path.resolve()}' does not exists.")

    with open(path, "r") as f:
        notebook = nbformat.reads(f.read(), nbformat.NO_CONVERT)
        exporter = PythonExporter()
        source, _ = exporter.from_notebook_node(notebook)

    with open(out_path, "w") as f:
        f.write(source)

# %% ../../nbs/098_Test_Dependencies.ipynb 17
@contextmanager
def generate_app_in_tmp() -> Generator[str, None, None]:
    with TemporaryDirectory() as d:
        src_path = Path(d) / "main.py"
        generate_app_src(src_path)
        with change_dir(d):
            import_str = f"{src_path.stem}:kafka_app"
            yield import_str



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/096_Meta.ipynb.

# %% auto 0
__all__ = ['TorF', 'T', 'patch', 'combine_params', 'delegates', 'use_parameters_of', 'filter_using_signature', 'export',
           'classcontextmanager']

# %% ../../nbs/096_Meta.ipynb 1
import builtins
import copy as cp
import functools
import inspect
import sys
import types
from functools import partial, wraps
from types import *
from typing import *

import docstring_parser

# %% ../../nbs/096_Meta.ipynb 4
def test_eq(a: Any, b: Any) -> None:
    "`test` that `a==b`"
    if a != b:
        raise ValueError(f"{a} != {b}")

# %% ../../nbs/096_Meta.ipynb 6
F = TypeVar("F", bound=Callable[..., Any])


def copy_func(f: Union[F, FunctionType]) -> Union[F, FunctionType]:
    "Copy a non-builtin function (NB `copy.copy` does not work for this)"
    if not isinstance(f, FunctionType):
        return cp.copy(f)
    fn = FunctionType(
        f.__code__, f.__globals__, f.__name__, f.__defaults__, f.__closure__
    )
    fn.__kwdefaults__ = f.__kwdefaults__
    fn.__dict__.update(f.__dict__)
    fn.__annotations__.update(f.__annotations__)
    fn.__qualname__ = f.__qualname__
    fn.__doc__ = f.__doc__
    return fn

# %% ../../nbs/096_Meta.ipynb 11
def patch_to(
    cls: Union[Type, Iterable[Type]], as_prop: bool = False, cls_method: bool = False
) -> Callable[[F], F]:
    "Decorator: add `f` to `cls`"
    if not isinstance(cls, (tuple, list)):
        cls = (cls,)  # type: ignore

    def _inner(f: F) -> F:
        for c_ in cls:
            nf = copy_func(f)
            nm = f.__name__
            # `functools.update_wrapper` when passing patched function to `Pipeline`, so we do it manually
            for o in functools.WRAPPER_ASSIGNMENTS:
                setattr(nf, o, getattr(f, o))
            nf.__qualname__ = f"{c_.__name__}.{nm}"
            if cls_method:
                setattr(c_, nm, MethodType(nf, c_))
            else:
                setattr(c_, nm, property(nf) if as_prop else nf)
        # Avoid clobbering existing functions
        # nosemgrep
        existing_func = globals().get(nm, builtins.__dict__.get(nm, None))
        return existing_func  # type: ignore

    return _inner

# %% ../../nbs/096_Meta.ipynb 22
def eval_type(
    t: Sequence, glb: Optional[Dict[str, Any]], loc: Optional[Mapping[str, object]]
) -> Any:
    "`eval` a type or collection of types, if needed, for annotations in py3.10+"
    if isinstance(t, str):
        if "|" in t:
            return Union[eval_type(tuple(t.split("|")), glb, loc)]
        # nosemgrep
        return eval(t, glb, loc)  # nosec B307:blacklist
    if isinstance(t, (tuple, list)):
        return type(t)([eval_type(c, glb, loc) for c in t])
    return t


def union2tuple(t) -> Tuple[Any, ...]:  # type: ignore
    if getattr(t, "__origin__", None) is Union:
        return t.__args__  # type: ignore

    if sys.version_info >= (3, 10):
        if isinstance(t, UnionType):
            return t.__args__

    return t  # type: ignore


def get_annotations_ex(
    obj: Union[FunctionType, Type, F],
    *,
    globals: Optional[Dict[str, Any]] = None,
    locals: Optional[Dict[str, Any]] = None,
) -> Tuple[Dict[str, Any], Union[Any, Dict[str, Any], None], Dict[str, Any]]:
    "Backport of py3.10 `get_annotations` that returns globals/locals"
    if isinstance(obj, type):
        obj_dict = getattr(obj, "__dict__", None)
        if obj_dict and hasattr(obj_dict, "get"):
            ann = obj_dict.get("__annotations__", None)
            if isinstance(ann, types.GetSetDescriptorType):
                ann = None
        else:
            ann = None

        obj_globals = None
        module_name = getattr(obj, "__module__", None)
        if module_name:
            module = sys.modules.get(module_name, None)
            if module:
                obj_globals = getattr(module, "__dict__", None)
        obj_locals = dict(vars(obj))
        unwrap = obj
    elif isinstance(obj, types.ModuleType):
        ann = getattr(obj, "__annotations__", None)
        obj_globals = getattr(obj, "__dict__")
        obj_locals, unwrap = None, None
    elif callable(obj):
        ann = getattr(obj, "__annotations__", None)
        obj_globals = getattr(obj, "__globals__", None)
        obj_locals, unwrap = None, obj  # type: ignore
    else:
        raise TypeError(f"{obj!r} is not a module, class, or callable.")

    if ann is None:
        ann = {}
    if not isinstance(ann, dict):
        raise ValueError(f"{obj!r}.__annotations__ is neither a dict nor None")
    if not ann:
        ann = {}

    if unwrap is not None:
        while True:
            if hasattr(unwrap, "__wrapped__"):
                unwrap = unwrap.__wrapped__
                continue
            if isinstance(unwrap, functools.partial):
                unwrap = unwrap.func  # type: ignore
                continue
            break
        if hasattr(unwrap, "__globals__"):
            obj_globals = unwrap.__globals__

    if globals is None:
        globals = obj_globals
    if locals is None:
        locals = obj_locals

    return dict(ann), globals, locals  # type: ignore

# %% ../../nbs/096_Meta.ipynb 23
def patch(  # type: ignore
    f: Optional[F] = None, *, as_prop: bool = False, cls_method: bool = False
):
    "Decorator: add `f` to the first parameter's class (based on f's type annotations)"
    if f is None:
        return partial(patch, as_prop=as_prop, cls_method=cls_method)
    ann, glb, loc = get_annotations_ex(f)
    cls = union2tuple(
        eval_type(ann.pop("cls") if cls_method else next(iter(ann.values())), glb, loc)
    )
    return patch_to(cls, as_prop=as_prop, cls_method=cls_method)(f)

# %% ../../nbs/096_Meta.ipynb 35
def _delegates_without_docs(
    to: Optional[F] = None,  # Delegatee
    keep: bool = False,  # Keep `kwargs` in decorated function?
    but: Optional[List[str]] = None,  # Exclude these parameters from signature
) -> Callable[[F], F]:
    "Decorator: replace `**kwargs` in signature with params from `to`"
    if but is None:
        but = []

    def _f(f: F) -> F:
        if to is None:
            to_f, from_f = f.__base__.__init__, f.__init__  # type: ignore
        else:
            to_f, from_f = to.__init__ if isinstance(to, type) else to, f  # type: ignore
        from_f = getattr(from_f, "__func__", from_f)
        to_f = getattr(to_f, "__func__", to_f)
        if hasattr(from_f, "__delwrap__"):
            return f
        sig = inspect.signature(from_f)
        sigd = dict(sig.parameters)
        if "kwargs" in sigd:
            k = sigd.pop("kwargs")
        else:
            k = None
        s2 = {
            k: v.replace(kind=inspect.Parameter.KEYWORD_ONLY)
            for k, v in inspect.signature(to_f).parameters.items()
            if v.default != inspect.Parameter.empty and k not in sigd and k not in but  # type: ignore
        }
        anno = {
            k: v
            for k, v in getattr(to_f, "__annotations__", {}).items()
            if k not in sigd and k not in but  # type: ignore
        }
        sigd.update(s2)
        if keep and k is not None:
            sigd["kwargs"] = k
        else:
            from_f.__delwrap__ = to_f
        from_f.__signature__ = sig.replace(parameters=list(sigd.values()))
        if hasattr(from_f, "__annotations__"):
            from_f.__annotations__.update(anno)
        return f

    return _f

# %% ../../nbs/096_Meta.ipynb 45
def _format_args(xs: List[docstring_parser.DocstringParam]) -> str:
    return "\nArgs:\n - " + "\n - ".join(
        [f"{x.arg_name} ({x.type_name}): {x.description}" for x in xs]
    )


def combine_params(f: F, o: Union[Type, Callable[..., Any]]) -> F:
    """Combines docstring arguments of a function and another object or function

    Args:
        f: destination functions where combined arguments will end up
        o: source function from which arguments are taken from

    Returns:
        Function f with augumented docstring including arguments from both functions/objects
    """
    src_params = docstring_parser.parse_from_object(o).params
    #     logger.info(f"combine_params(): source:{_format_args(src_params)}")
    docs = docstring_parser.parse_from_object(f)
    #     logger.info(f"combine_params(): destination:{_format_args(docs.params)}")
    dst_params_names = [p.arg_name for p in docs.params]

    combined_params = docs.params + [
        x for x in src_params if not x.arg_name in dst_params_names
    ]
    #     logger.info(f"combine_params(): combined:{_format_args(combined_params)}")

    docs.meta = [
        x for x in docs.meta if not isinstance(x, docstring_parser.DocstringParam)
    ] + combined_params  # type: ignore

    f.__doc__ = docstring_parser.compose(
        docs, style=docstring_parser.DocstringStyle.GOOGLE
    )
    return f

# %% ../../nbs/096_Meta.ipynb 47
def delegates(
    o: Union[Type, Callable[..., Any]],
    keep: bool = False,
    but: Optional[List[str]] = None,
) -> Callable[[F], F]:
    """Delegates keyword agruments from o to the function the decorator is applied to

    Args:
        o: object (class or function) with default kwargs
        keep: Keep `kwargs` in decorated function?
        but: argument names not to include
    """

    def _inner(f: F, keep: bool = keep, but: Optional[List[str]] = but) -> F:
        def _combine_params(o: Union[Type, Callable[..., Any]]) -> Callable[[F], F]:
            def __combine_params(f: F, o: Union[Type, Callable[..., Any]] = o) -> F:
                return combine_params(f=f, o=o)

            return __combine_params

        @_combine_params(o)  # type: ignore
        @_delegates_without_docs(o, keep=keep, but=but)  # type: ignore
        @wraps(f)
        def _f(*args: Any, **kwargs: Any) -> Any:
            return f(*args, **kwargs)

        return _f

    return _inner

# %% ../../nbs/096_Meta.ipynb 64
def use_parameters_of(
    o: Union[Type, Callable[..., Any]], **kwargs: Dict[str, Any]
) -> Dict[str, Any]:
    """Restrict parameters passwed as keyword arguments to parameters from the signature of ``o``

    Args:
        o: object or callable which signature is used for restricting keyword arguments
        kwargs: keyword arguments

    Returns:
        restricted keyword arguments

    """
    allowed_keys = set(inspect.signature(o).parameters.keys())
    return {k: v for k, v in kwargs.items() if k in allowed_keys}

# %% ../../nbs/096_Meta.ipynb 66
def filter_using_signature(f: Callable, **kwargs: Dict[str, Any]) -> Dict[str, Any]:
    """todo: write docs"""
    param_names = list(inspect.signature(f).parameters.keys())
    return {k: v for k, v in kwargs.items() if k in param_names}

# %% ../../nbs/096_Meta.ipynb 68
TorF = TypeVar("TorF", Type, Callable[..., Any])


def export(module_name: str) -> Callable[[TorF], TorF]:
    def _inner(o: TorF, module_name: str = module_name) -> TorF:
        o.__module__ = module_name
        return o

    return _inner

# %% ../../nbs/096_Meta.ipynb 71
T = TypeVar("T")


def classcontextmanager(name: str = "lifecycle") -> Callable[[Type[T]], Type[T]]:
    def _classcontextmanager(cls: Type[T], name: str = name) -> Type[T]:
        if not hasattr(cls, name):
            raise ValueError

        @patch
        def __enter__(self: cls) -> Any:  # type: ignore
            if not hasattr(self, "_lifecycle_ctx"):
                self._lifecycle_ctx = []  # type: ignore

            self._lifecycle_ctx.append(getattr(self, name)())  # type: ignore
            return self._lifecycle_ctx[-1].__enter__()  # type: ignore

        @patch
        def __exit__(self: cls, *args: Any) -> None:  # type: ignore
            self._lifecycle_ctx.pop(-1).__exit__(*args)  # type: ignore

        return cls

    return _classcontextmanager

# %% ../../nbs/096_Meta.ipynb 74
def _get_default_kwargs_from_sig(f: F, **kwargs: Any) -> Dict[str, Any]:
    """
    Get default values for function **kwargs

    Args:
        f: Function to extract default values from

    Returns:
        Dict of default values of function f **kwargs
    """
    defaults = {
        k: v.default
        for k, v in inspect.signature(f).parameters.items()
        if v.default != inspect._empty
    }
    defaults.update(kwargs)
    return defaults



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/001_InMemoryBroker.ipynb.

# %% auto 0
__all__ = ['logger', 'KafkaRecord', 'KafkaPartition', 'KafkaTopic', 'split_list', 'GroupMetadata', 'InMemoryBroker',
           'InMemoryConsumer', 'InMemoryProducer']

# %% ../../nbs/001_InMemoryBroker.ipynb 1
import asyncio
import copy
import hashlib
import inspect
import random
import string

import uuid
from collections import namedtuple
from contextlib import contextmanager
from dataclasses import dataclass
from typing import *

from aiokafka import AIOKafkaConsumer, AIOKafkaProducer
from aiokafka.structs import ConsumerRecord, RecordMetadata, TopicPartition

import fastkafka._application.app
import fastkafka._components.aiokafka_consumer_loop
import fastkafka._components.aiokafka_producer_manager
from .._components.logger import get_logger
from fastkafka._components.meta import (
    _get_default_kwargs_from_sig,
    classcontextmanager,
    copy_func,
    delegates,
    patch,
)

# %% ../../nbs/001_InMemoryBroker.ipynb 3
logger = get_logger(__name__)

# %% ../../nbs/001_InMemoryBroker.ipynb 6
@dataclass
class KafkaRecord:
    topic: str = ""
    partition: int = 0
    key: Optional[bytes] = None
    value: bytes = b""
    offset: int = 0

# %% ../../nbs/001_InMemoryBroker.ipynb 7
class KafkaPartition:
    def __init__(self, *, partition: int, topic: str):
        self.partition = partition
        self.topic = topic
        self.messages: List[KafkaRecord] = list()

    def write(self, value: bytes, key: Optional[bytes] = None) -> RecordMetadata:  # type: ignore
        record = KafkaRecord(
            topic=self.topic,
            partition=self.partition,
            value=value,
            key=key,
            offset=len(self.messages),
        )
        record_meta = RecordMetadata(
            topic=self.topic,
            partition=self.partition,
            topic_partition=TopicPartition(topic=self.topic, partition=self.partition),
            offset=len(self.messages),
            timestamp=1680602752070,
            timestamp_type=0,
            log_start_offset=0,
        )
        self.messages.append(record)
        return record_meta

    def read(self, offset: int) -> Tuple[List[KafkaRecord], int]:
        return self.messages[offset:], len(self.messages)

    def latest_offset(self) -> int:
        return len(self.messages)

# %% ../../nbs/001_InMemoryBroker.ipynb 11
class KafkaTopic:
    def __init__(self, topic: str, num_partitions: int = 1):
        self.topic = topic
        self.num_partitions = num_partitions
        self.partitions: List[KafkaPartition] = [
            KafkaPartition(topic=topic, partition=partition_index)
            for partition_index in range(num_partitions)
        ]

    def read(  # type: ignore
        self, partition: int, offset: int
    ) -> Tuple[TopicPartition, List[KafkaRecord], int]:
        topic_partition = TopicPartition(topic=self.topic, partition=partition)
        records, offset = self.partitions[partition].read(offset)
        return topic_partition, records, offset

    def write_with_partition(  # type: ignore
        self,
        value: bytes,
        partition: int,
    ) -> RecordMetadata:
        return self.partitions[partition].write(value)

    def write_with_key(self, value: bytes, key: bytes) -> RecordMetadata:  # type: ignore
        partition = int(hashlib.sha256(key).hexdigest(), 16) % self.num_partitions
        return self.partitions[partition].write(value, key=key)

    def write(  # type: ignore
        self,
        value: bytes,
        *,
        key: Optional[bytes] = None,
        partition: Optional[int] = None,
    ) -> RecordMetadata:
        if partition is not None:
            return self.write_with_partition(value, partition)

        if key is not None:
            return self.write_with_key(value, key)

        partition = random.randint(0, self.num_partitions - 1)  # nosec
        return self.write_with_partition(value, partition)

    def latest_offset(self, partition: int) -> int:
        return self.partitions[partition].latest_offset()

# %% ../../nbs/001_InMemoryBroker.ipynb 17
def split_list(list_to_split: List[Any], split_size: int) -> List[List[Any]]:
    return [
        list_to_split[start_index : start_index + split_size]
        for start_index in range(0, len(list_to_split), split_size)
    ]

# %% ../../nbs/001_InMemoryBroker.ipynb 19
class GroupMetadata:
    def __init__(self, num_partitions: int):
        self.num_partitions = num_partitions
        self.partitions_offsets: Dict[int, int] = {}
        self.consumer_ids: List[uuid.UUID] = list()
        self.partition_assignments: Dict[uuid.UUID, List[int]] = {}

    def subscribe(self, consumer_id: uuid.UUID) -> None:
        self.consumer_ids.append(consumer_id)
        self.rebalance()

    def unsubscribe(self, consumer_id: uuid.UUID) -> None:
        self.consumer_ids.remove(consumer_id)
        self.rebalance()

    def rebalance(self) -> None:
        if len(self.consumer_ids) == 0:
            self.partition_assignments = {}
        else:
            partitions_per_actor = self.num_partitions // len(self.consumer_ids)
            if self.num_partitions % len(self.consumer_ids) != 0:
                partitions_per_actor += 1
            self.assign_partitions(partitions_per_actor)

    def assign_partitions(self, partitions_per_actor: int) -> None:
        partitions = [i for i in range(self.num_partitions)]

        partitions_split = split_list(partitions, partitions_per_actor)
        self.partition_assignments = {
            self.consumer_ids[i]: partition_split
            for i, partition_split in enumerate(partitions_split)
        }

    def get_partitions(
        self, consumer_id: uuid.UUID
    ) -> Tuple[List[int], Dict[int, Optional[int]]]:
        partition_assignments = self.partition_assignments.get(consumer_id, [])
        partition_offsets_assignments = {
            partition: self.partitions_offsets.get(partition, None)
            for partition in partition_assignments
        }
        return partition_assignments, partition_offsets_assignments

    def set_offset(self, partition: int, offset: int) -> None:
        self.partitions_offsets[partition] = offset

# %% ../../nbs/001_InMemoryBroker.ipynb 22
@classcontextmanager()
class InMemoryBroker:
    def __init__(
        self,
        num_partitions: int = 1,
    ):
        self.num_partitions = num_partitions
        self.topics: Dict[Tuple[str, str], KafkaTopic] = {}
        self.topic_groups: Dict[Tuple[str, str, str], GroupMetadata] = {}
        self.is_started: bool = False

    def connect(self) -> uuid.UUID:
        return uuid.uuid4()

    def dissconnect(self, consumer_id: uuid.UUID) -> None:
        pass

    def subscribe(
        self, bootstrap_server: str, topic: str, group: str, consumer_id: uuid.UUID
    ) -> None:
        raise NotImplementedError()

    def unsubscribe(
        self, bootstrap_server: str, topic: str, group: str, consumer_id: uuid.UUID
    ) -> None:
        raise NotImplementedError()

    def read(  # type: ignore
        self,
        *,
        bootstrap_server: str,
        topic: str,
        group: str,
        consumer_id: uuid.UUID,
        auto_offset_reset: str,
    ) -> Dict[TopicPartition, List[KafkaRecord]]:
        raise NotImplementedError()

    def write(  # type: ignore
        self,
        *,
        bootstrap_server: str,
        topic: str,
        value: bytes,
        key: Optional[bytes] = None,
        partition: Optional[int] = None,
    ) -> RecordMetadata:
        raise NotImplementedError()

    @contextmanager
    def lifecycle(self) -> Iterator["InMemoryBroker"]:
        raise NotImplementedError()

    async def _start(self) -> str:
        logger.info("InMemoryBroker._start() called")
        self.__enter__()  # type: ignore
        return "localbroker:0"

    async def _stop(self) -> None:
        logger.info("InMemoryBroker._stop() called")
        self.__exit__(None, None, None)  # type: ignore

# %% ../../nbs/001_InMemoryBroker.ipynb 23
@patch
def subscribe(
    self: InMemoryBroker,
    bootstrap_server: str,
    topic: str,
    group: str,
    consumer_id: uuid.UUID,
) -> None:
    if (bootstrap_server, topic) not in self.topics:
        self.topics[(bootstrap_server, topic)] = KafkaTopic(
            topic=topic, num_partitions=self.num_partitions
        )

    group_meta = self.topic_groups.get(
        (bootstrap_server, topic, group), GroupMetadata(self.num_partitions)
    )
    group_meta.subscribe(consumer_id)
    self.topic_groups[(bootstrap_server, topic, group)] = group_meta


@patch
def unsubscribe(
    self: InMemoryBroker,
    bootstrap_server: str,
    topic: str,
    group: str,
    consumer_id: uuid.UUID,
) -> None:
    self.topic_groups[(bootstrap_server, topic, group)].unsubscribe(consumer_id)

# %% ../../nbs/001_InMemoryBroker.ipynb 25
@patch
def write(  # type: ignore
    self: InMemoryBroker,
    *,
    bootstrap_server: str,
    topic: str,
    value: bytes,
    key: Optional[bytes] = None,
    partition: Optional[int] = None,
) -> RecordMetadata:
    if (bootstrap_server, topic) not in self.topics:
        self.topics[(bootstrap_server, topic)] = KafkaTopic(
            topic=topic, num_partitions=self.num_partitions
        )

    return self.topics[(bootstrap_server, topic)].write(
        value, key=key, partition=partition
    )

# %% ../../nbs/001_InMemoryBroker.ipynb 27
@patch
def read(  # type: ignore
    self: InMemoryBroker,
    *,
    bootstrap_server: str,
    topic: str,
    group: str,
    consumer_id: uuid.UUID,
    auto_offset_reset: str,
) -> Dict[TopicPartition, List[KafkaRecord]]:
    group_meta = self.topic_groups[(bootstrap_server, topic, group)]
    partitions, offsets = group_meta.get_partitions(consumer_id)

    if len(partitions) == 0:
        return {}

    partitions_data = {}

    for partition in partitions:
        offset = offsets[partition]

        if offset is None:
            offset = (
                self.topics[(bootstrap_server, topic)].latest_offset(partition)
                if auto_offset_reset == "latest"
                else 0
            )

        topic_partition, data, offset = self.topics[(bootstrap_server, topic)].read(
            partition, offset
        )

        partitions_data[topic_partition] = data
        group_meta.set_offset(partition, offset)

    return partitions_data

# %% ../../nbs/001_InMemoryBroker.ipynb 34
# InMemoryConsumer
class InMemoryConsumer:
    def __init__(
        self,
        broker: InMemoryBroker,
    ) -> None:
        self.broker = broker
        self._id: Optional[uuid.UUID] = None
        self._auto_offset_reset: str = "latest"
        self._group_id: Optional[str] = None
        self._topics: List[str] = list()
        self._bootstrap_servers = ""

    @delegates(AIOKafkaConsumer)
    def __call__(self, **kwargs: Any) -> "InMemoryConsumer":
        defaults = _get_default_kwargs_from_sig(InMemoryConsumer.__call__, **kwargs)
        consume_copy = InMemoryConsumer(self.broker)
        consume_copy._auto_offset_reset = defaults["auto_offset_reset"]
        consume_copy._bootstrap_servers = defaults["bootstrap_servers"]
        consume_copy._group_id = (
            defaults["group_id"]
            if defaults["group_id"] is not None
            else "".join(random.choices(string.ascii_letters, k=10))  # nosec
        )
        return consume_copy

    @delegates(AIOKafkaConsumer.start)
    async def start(self, **kwargs: Any) -> None:
        pass

    @delegates(AIOKafkaConsumer.stop)
    async def stop(self, **kwargs: Any) -> None:
        pass

    @delegates(AIOKafkaConsumer.subscribe)
    def subscribe(self, topics: List[str], **kwargs: Any) -> None:
        raise NotImplementedError()

    @delegates(AIOKafkaConsumer.getmany)
    async def getmany(  # type: ignore
        self, **kwargs: Any
    ) -> Dict[TopicPartition, List[ConsumerRecord]]:
        raise NotImplementedError()

# %% ../../nbs/001_InMemoryBroker.ipynb 37
@patch
@delegates(AIOKafkaConsumer.start)
async def start(self: InMemoryConsumer, **kwargs: Any) -> None:
    logger.info("AIOKafkaConsumer patched start() called()")
    if self._id is not None:
        raise RuntimeError(
            "Consumer start() already called! Run consumer stop() before running start() again"
        )
    self._id = self.broker.connect()

# %% ../../nbs/001_InMemoryBroker.ipynb 40
@patch  # type: ignore
@delegates(AIOKafkaConsumer.subscribe)
def subscribe(self: InMemoryConsumer, topics: List[str], **kwargs: Any) -> None:
    logger.info("AIOKafkaConsumer patched subscribe() called")
    if self._id is None:
        raise RuntimeError("Consumer start() not called! Run consumer start() first")
    logger.info(f"AIOKafkaConsumer.subscribe(), subscribing to: {topics}")
    for topic in topics:
        self.broker.subscribe(
            bootstrap_server=self._bootstrap_servers,
            consumer_id=self._id,
            topic=topic,
            group=self._group_id,  # type: ignore
        )
        self._topics.append(topic)

# %% ../../nbs/001_InMemoryBroker.ipynb 43
@patch
@delegates(AIOKafkaConsumer.stop)
async def stop(self: InMemoryConsumer, **kwargs: Any) -> None:
    logger.info("AIOKafkaConsumer patched stop() called")
    if self._id is None:
        raise RuntimeError("Consumer start() not called! Run consumer start() first")
    for topic in self._topics:
        self.broker.unsubscribe(
            bootstrap_server=self._bootstrap_servers,
            topic=topic,
            group=self._group_id,  # type: ignore
            consumer_id=self._id,
        )

# %% ../../nbs/001_InMemoryBroker.ipynb 46
@patch
@delegates(AIOKafkaConsumer.getmany)
async def getmany(  # type: ignore
    self: InMemoryConsumer, **kwargs: Any
) -> Dict[TopicPartition, List[ConsumerRecord]]:
    for topic in self._topics:
        return self.broker.read(
            bootstrap_server=self._bootstrap_servers,
            topic=topic,
            consumer_id=self._id,  # type: ignore
            group=self._group_id,  # type: ignore
            auto_offset_reset=self._auto_offset_reset,
        )

# %% ../../nbs/001_InMemoryBroker.ipynb 49
class InMemoryProducer:
    def __init__(self, broker: InMemoryBroker, **kwargs: Any) -> None:
        self.broker = broker
        self.id: Optional[uuid.UUID] = None
        self._bootstrap_servers = ""

    @delegates(AIOKafkaProducer)
    def __call__(self, **kwargs: Any) -> "InMemoryProducer":
        defaults = _get_default_kwargs_from_sig(InMemoryConsumer.__call__, **kwargs)
        producer_copy = InMemoryProducer(self.broker)
        producer_copy._bootstrap_servers = defaults["bootstrap_servers"]
        return producer_copy

    @delegates(AIOKafkaProducer.start)
    async def start(self, **kwargs: Any) -> None:
        raise NotImplementedError()

    @delegates(AIOKafkaProducer.stop)
    async def stop(self, **kwargs: Any) -> None:
        raise NotImplementedError()

    @delegates(AIOKafkaProducer.send)
    async def send(  # type: ignore
        self: AIOKafkaProducer,
        topic: str,
        msg: bytes,
        key: Optional[bytes] = None,
        **kwargs: Any,
    ):
        raise NotImplementedError()

# %% ../../nbs/001_InMemoryBroker.ipynb 52
@patch  # type: ignore
@delegates(AIOKafkaProducer.start)
async def start(self: InMemoryProducer, **kwargs: Any) -> None:
    logger.info("AIOKafkaProducer patched start() called()")
    if self.id is not None:
        raise RuntimeError(
            "Producer start() already called! Run producer stop() before running start() again"
        )
    self.id = self.broker.connect()

# %% ../../nbs/001_InMemoryBroker.ipynb 55
@patch  # type: ignore
@delegates(AIOKafkaProducer.stop)
async def stop(self: InMemoryProducer, **kwargs: Any) -> None:
    logger.info("AIOKafkaProducer patched stop() called")
    if self.id is None:
        raise RuntimeError("Producer start() not called! Run producer start() first")

# %% ../../nbs/001_InMemoryBroker.ipynb 58
@patch
@delegates(AIOKafkaProducer.send)
async def send(  # type: ignore
    self: InMemoryProducer,
    topic: str,
    msg: bytes,
    key: Optional[bytes] = None,
    partition: Optional[int] = None,
    **kwargs: Any,
):  # asyncio.Task[RecordMetadata]
    if self.id is None:
        raise RuntimeError("Producer start() not called! Run producer start() first")

    record = self.broker.write(
        bootstrap_server=self._bootstrap_servers,
        topic=topic,
        value=msg,
        key=key,
        partition=partition,
    )

    async def _f(record: ConsumerRecord = record) -> RecordMetadata:  # type: ignore
        return record

    return asyncio.create_task(_f())

# %% ../../nbs/001_InMemoryBroker.ipynb 61
@patch
@contextmanager
def lifecycle(self: InMemoryBroker) -> Iterator[InMemoryBroker]:
    logger.info(
        "InMemoryBroker._patch_consumers_and_producers(): Patching consumers and producers!"
    )
    try:
        logger.info("InMemoryBroker starting")

        old_consumer_app = fastkafka._application.app.AIOKafkaConsumer
        old_producer_app = fastkafka._application.app.AIOKafkaProducer
        old_consumer_loop = (
            fastkafka._components.aiokafka_consumer_loop.AIOKafkaConsumer
        )
        old_producer_manager = (
            fastkafka._components.aiokafka_producer_manager.AIOKafkaProducer
        )

        fastkafka._application.app.AIOKafkaConsumer = InMemoryConsumer(self)
        fastkafka._application.app.AIOKafkaProducer = InMemoryProducer(self)
        fastkafka._components.aiokafka_consumer_loop.AIOKafkaConsumer = (
            InMemoryConsumer(self)
        )
        fastkafka._components.aiokafka_producer_manager.AIOKafkaProducer = (
            InMemoryProducer(self)
        )

        self.is_started = True
        yield self
    finally:
        logger.info("InMemoryBroker stopping")

        fastkafka._application.app.AIOKafkaConsumer = old_consumer_app
        fastkafka._application.app.AIOKafkaProducer = old_producer_app
        fastkafka._components.aiokafka_consumer_loop.AIOKafkaConsumer = (
            old_consumer_loop
        )
        fastkafka._components.aiokafka_producer_manager.AIOKafkaProducer = (
            old_producer_manager
        )

        self.is_started = False



__version__ = "0.4.0rc1"
# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/010_Application_export.ipynb.

# %% auto 0
__all__ = ['dummy']

# %% ../nbs/010_Application_export.ipynb 1
from ._application.app import FastKafka
from ._components.producer_decorator import KafkaEvent

__all__ = [
    "FastKafka",
    "KafkaEvent",
]

# %% ../nbs/010_Application_export.ipynb 2
def dummy() -> None:
    pass


dummy.__module__ = "_dummy"



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/fastkafka',
                'doc_host': 'https://airtai.github.io',
                'git_url': 'https://github.com/airtai/fastkafka',
                'lib_path': 'fastkafka'},
  'syms': { 'fastkafka._application.app': { 'fastkafka._application.app.AwaitedMock': ( 'fastkafka.html#awaitedmock',
                                                                                        'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.AwaitedMock.__init__': ( 'fastkafka.html#awaitedmock.__init__',
                                                                                                 'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.AwaitedMock._await_for': ( 'fastkafka.html#awaitedmock._await_for',
                                                                                                   'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka': ( 'fastkafka.html#fastkafka',
                                                                                      'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.__aenter__': ( 'fastkafka.html#fastkafka.__aenter__',
                                                                                                 'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.__aexit__': ( 'fastkafka.html#fastkafka.__aexit__',
                                                                                                'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.__init__': ( 'fastkafka.html#fastkafka.__init__',
                                                                                               'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._populate_bg_tasks': ( 'fastkafka.html#fastkafka._populate_bg_tasks',
                                                                                                         'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._populate_consumers': ( 'fastkafka.html#fastkafka._populate_consumers',
                                                                                                          'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._populate_producers': ( 'fastkafka.html#fastkafka._populate_producers',
                                                                                                          'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._set_bootstrap_servers': ( 'fastkafka.html#fastkafka._set_bootstrap_servers',
                                                                                                             'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._shutdown_bg_tasks': ( 'fastkafka.html#fastkafka._shutdown_bg_tasks',
                                                                                                         'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._shutdown_consumers': ( 'fastkafka.html#fastkafka._shutdown_consumers',
                                                                                                          'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._shutdown_producers': ( 'fastkafka.html#fastkafka._shutdown_producers',
                                                                                                          'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._start': ( 'fastkafka.html#fastkafka._start',
                                                                                             'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka._stop': ( 'fastkafka.html#fastkafka._stop',
                                                                                            'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.benchmark': ( 'fastkafka.html#fastkafka.benchmark',
                                                                                                'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.consumes': ( 'fastkafka.html#fastkafka.consumes',
                                                                                               'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.create_docs': ( 'fastkafka.html#fastkafka.create_docs',
                                                                                                  'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.create_mocks': ( 'fastkafka.html#fastkafka.create_mocks',
                                                                                                   'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.get_topics': ( 'fastkafka.html#fastkafka.get_topics',
                                                                                                 'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.is_started': ( 'fastkafka.html#fastkafka.is_started',
                                                                                                 'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.produces': ( 'fastkafka.html#fastkafka.produces',
                                                                                               'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.run_in_background': ( 'fastkafka.html#fastkafka.run_in_background',
                                                                                                        'fastkafka/_application/app.py'),
                                            'fastkafka._application.app.FastKafka.set_kafka_broker': ( 'fastkafka.html#fastkafka.set_kafka_broker',
                                                                                                       'fastkafka/_application/app.py'),
                                            'fastkafka._application.app._create_producer': ( 'fastkafka.html#_create_producer',
                                                                                             'fastkafka/_application/app.py'),
                                            'fastkafka._application.app._get_contact_info': ( 'fastkafka.html#_get_contact_info',
                                                                                              'fastkafka/_application/app.py'),
                                            'fastkafka._application.app._get_kafka_brokers': ( 'fastkafka.html#_get_kafka_brokers',
                                                                                               'fastkafka/_application/app.py'),
                                            'fastkafka._application.app._get_kafka_config': ( 'fastkafka.html#_get_kafka_config',
                                                                                              'fastkafka/_application/app.py'),
                                            'fastkafka._application.app._get_topic_name': ( 'fastkafka.html#_get_topic_name',
                                                                                            'fastkafka/_application/app.py')},
            'fastkafka._application.tester': { 'fastkafka._application.tester.Tester': ( 'tester.html#tester',
                                                                                         'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.__aenter__': ( 'tester.html#tester.__aenter__',
                                                                                                    'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.__aexit__': ( 'tester.html#tester.__aexit__',
                                                                                                   'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.__init__': ( 'tester.html#tester.__init__',
                                                                                                  'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester._create_ctx': ( 'tester.html#tester._create_ctx',
                                                                                                     'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester._start_tester': ( 'tester.html#tester._start_tester',
                                                                                                       'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester._stop_tester': ( 'tester.html#tester._stop_tester',
                                                                                                      'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.create_mirrors': ( 'tester.html#tester.create_mirrors',
                                                                                                        'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.using_local_kafka': ( 'tester.html#tester.using_local_kafka',
                                                                                                           'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.Tester.using_local_redpanda': ( 'tester.html#tester.using_local_redpanda',
                                                                                                              'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.mirror_consumer': ( 'tester.html#mirror_consumer',
                                                                                                  'fastkafka/_application/tester.py'),
                                               'fastkafka._application.tester.mirror_producer': ( 'tester.html#mirror_producer',
                                                                                                  'fastkafka/_application/tester.py')},
            'fastkafka._components.aiokafka_consumer_loop': { 'fastkafka._components.aiokafka_consumer_loop._aiokafka_consumer_loop': ( 'consumerloop.html#_aiokafka_consumer_loop',
                                                                                                                                        'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop._create_safe_callback': ( 'consumerloop.html#_create_safe_callback',
                                                                                                                                      'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop._decode_streamed_msgs': ( 'consumerloop.html#_decode_streamed_msgs',
                                                                                                                                      'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop._prepare_callback': ( 'consumerloop.html#_prepare_callback',
                                                                                                                                  'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop._stream_msgs': ( 'consumerloop.html#_stream_msgs',
                                                                                                                             'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop._streamed_records': ( 'consumerloop.html#_streamed_records',
                                                                                                                                  'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop.aiokafka_consumer_loop': ( 'consumerloop.html#aiokafka_consumer_loop',
                                                                                                                                       'fastkafka/_components/aiokafka_consumer_loop.py'),
                                                              'fastkafka._components.aiokafka_consumer_loop.sanitize_kafka_config': ( 'consumerloop.html#sanitize_kafka_config',
                                                                                                                                      'fastkafka/_components/aiokafka_consumer_loop.py')},
            'fastkafka._components.aiokafka_producer_manager': { 'fastkafka._components.aiokafka_producer_manager.AIOKafkaProducerManager': ( 'producermanager.html#aiokafkaproducermanager',
                                                                                                                                              'fastkafka/_components/aiokafka_producer_manager.py'),
                                                                 'fastkafka._components.aiokafka_producer_manager.AIOKafkaProducerManager.__init__': ( 'producermanager.html#aiokafkaproducermanager.__init__',
                                                                                                                                                       'fastkafka/_components/aiokafka_producer_manager.py'),
                                                                 'fastkafka._components.aiokafka_producer_manager.AIOKafkaProducerManager.send': ( 'producermanager.html#aiokafkaproducermanager.send',
                                                                                                                                                   'fastkafka/_components/aiokafka_producer_manager.py'),
                                                                 'fastkafka._components.aiokafka_producer_manager.AIOKafkaProducerManager.start': ( 'producermanager.html#aiokafkaproducermanager.start',
                                                                                                                                                    'fastkafka/_components/aiokafka_producer_manager.py'),
                                                                 'fastkafka._components.aiokafka_producer_manager.AIOKafkaProducerManager.stop': ( 'producermanager.html#aiokafkaproducermanager.stop',
                                                                                                                                                   'fastkafka/_components/aiokafka_producer_manager.py'),
                                                                 'fastkafka._components.aiokafka_producer_manager._aiokafka_producer_manager': ( 'producermanager.html#_aiokafka_producer_manager',
                                                                                                                                                 'fastkafka/_components/aiokafka_producer_manager.py')},
            'fastkafka._components.asyncapi': { 'fastkafka._components.asyncapi.APIKeyLocation': ( 'asyncapi.html#apikeylocation',
                                                                                                   'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.ContactInfo': ( 'asyncapi.html#contactinfo',
                                                                                                'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaBroker': ( 'asyncapi.html#kafkabroker',
                                                                                                'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaBroker.dict': ( 'asyncapi.html#kafkabroker.dict',
                                                                                                     'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaBroker.json': ( 'asyncapi.html#kafkabroker.json',
                                                                                                     'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaBrokers': ( 'asyncapi.html#kafkabrokers',
                                                                                                 'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaMessage': ( 'asyncapi.html#kafkamessage',
                                                                                                 'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaMessage.Config': ( 'asyncapi.html#kafkamessage.config',
                                                                                                        'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.KafkaServiceInfo': ( 'asyncapi.html#kafkaserviceinfo',
                                                                                                     'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.SecuritySchema': ( 'asyncapi.html#securityschema',
                                                                                                   'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.SecuritySchema.__init__': ( 'asyncapi.html#securityschema.__init__',
                                                                                                            'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.SecuritySchema.dict': ( 'asyncapi.html#securityschema.dict',
                                                                                                        'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.SecuritySchema.json': ( 'asyncapi.html#securityschema.json',
                                                                                                        'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.SecurityType': ( 'asyncapi.html#securitytype',
                                                                                                 'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._add_example_to_msg_definitions': ( 'asyncapi.html#_add_example_to_msg_definitions',
                                                                                                                    'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._generate_async_docs': ( 'asyncapi.html#_generate_async_docs',
                                                                                                         'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._generate_async_spec': ( 'asyncapi.html#_generate_async_spec',
                                                                                                         'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_asyncapi_schema': ( 'asyncapi.html#_get_asyncapi_schema',
                                                                                                         'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_channels_schema': ( 'asyncapi.html#_get_channels_schema',
                                                                                                         'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_components_schema': ( 'asyncapi.html#_get_components_schema',
                                                                                                           'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_example': ( 'asyncapi.html#_get_example',
                                                                                                 'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_kafka_msg_classes': ( 'asyncapi.html#_get_kafka_msg_classes',
                                                                                                           'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_kafka_msg_definitions': ( 'asyncapi.html#_get_kafka_msg_definitions',
                                                                                                               'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_msg_cls_for_consumer': ( 'asyncapi.html#_get_msg_cls_for_consumer',
                                                                                                              'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_msg_cls_for_producer': ( 'asyncapi.html#_get_msg_cls_for_producer',
                                                                                                              'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_msg_definitions_with_examples': ( 'asyncapi.html#_get_msg_definitions_with_examples',
                                                                                                                       'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_security_schemes': ( 'asyncapi.html#_get_security_schemes',
                                                                                                          'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_servers_schema': ( 'asyncapi.html#_get_servers_schema',
                                                                                                        'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi._get_topic_dict': ( 'asyncapi.html#_get_topic_dict',
                                                                                                    'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.export_async_spec': ( 'asyncapi.html#export_async_spec',
                                                                                                      'fastkafka/_components/asyncapi.py'),
                                                'fastkafka._components.asyncapi.yaml_file_cmp': ( 'asyncapi.html#yaml_file_cmp',
                                                                                                  'fastkafka/_components/asyncapi.py')},
            'fastkafka._components.benchmarking': { 'fastkafka._components.benchmarking._benchmark': ( 'benchmarking.html#_benchmark',
                                                                                                       'fastkafka/_components/benchmarking.py')},
            'fastkafka._components.docs_dependencies': { 'fastkafka._components.docs_dependencies._check_npm': ( 'docs_dependencies.html#_check_npm',
                                                                                                                 'fastkafka/_components/docs_dependencies.py'),
                                                         'fastkafka._components.docs_dependencies._check_npm_with_local': ( 'docs_dependencies.html#_check_npm_with_local',
                                                                                                                            'fastkafka/_components/docs_dependencies.py'),
                                                         'fastkafka._components.docs_dependencies._install_docs_npm_deps': ( 'docs_dependencies.html#_install_docs_npm_deps',
                                                                                                                             'fastkafka/_components/docs_dependencies.py'),
                                                         'fastkafka._components.docs_dependencies._install_node': ( 'docs_dependencies.html#_install_node',
                                                                                                                    'fastkafka/_components/docs_dependencies.py')},
            'fastkafka._components.helpers': { 'fastkafka._components.helpers.ImportFromStringError': ( 'internal_helpers.html#importfromstringerror',
                                                                                                        'fastkafka/_components/helpers.py'),
                                               'fastkafka._components.helpers._import_from_string': ( 'internal_helpers.html#_import_from_string',
                                                                                                      'fastkafka/_components/helpers.py'),
                                               'fastkafka._components.helpers.change_dir': ( 'internal_helpers.html#change_dir',
                                                                                             'fastkafka/_components/helpers.py'),
                                               'fastkafka._components.helpers.in_notebook': ( 'internal_helpers.html#in_notebook',
                                                                                              'fastkafka/_components/helpers.py')},
            'fastkafka._components.logger': { 'fastkafka._components.logger.get_default_logger_configuration': ( 'logger.html#get_default_logger_configuration',
                                                                                                                 'fastkafka/_components/logger.py'),
                                              'fastkafka._components.logger.get_logger': ( 'logger.html#get_logger',
                                                                                           'fastkafka/_components/logger.py'),
                                              'fastkafka._components.logger.set_level': ( 'logger.html#set_level',
                                                                                          'fastkafka/_components/logger.py'),
                                              'fastkafka._components.logger.supress_timestamps': ( 'logger.html#supress_timestamps',
                                                                                                   'fastkafka/_components/logger.py')},
            'fastkafka._components.meta': { 'fastkafka._components.meta._delegates_without_docs': ( 'meta.html#_delegates_without_docs',
                                                                                                    'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta._format_args': ( 'meta.html#_format_args',
                                                                                         'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta._get_default_kwargs_from_sig': ( 'meta.html#_get_default_kwargs_from_sig',
                                                                                                         'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.classcontextmanager': ( 'meta.html#classcontextmanager',
                                                                                                'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.combine_params': ( 'meta.html#combine_params',
                                                                                           'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.copy_func': ( 'meta.html#copy_func',
                                                                                      'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.delegates': ( 'meta.html#delegates',
                                                                                      'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.eval_type': ( 'meta.html#eval_type',
                                                                                      'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.export': ('meta.html#export', 'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.filter_using_signature': ( 'meta.html#filter_using_signature',
                                                                                                   'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.get_annotations_ex': ( 'meta.html#get_annotations_ex',
                                                                                               'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.patch': ('meta.html#patch', 'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.patch_to': ('meta.html#patch_to', 'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.test_eq': ('meta.html#test_eq', 'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.union2tuple': ( 'meta.html#union2tuple',
                                                                                        'fastkafka/_components/meta.py'),
                                            'fastkafka._components.meta.use_parameters_of': ( 'meta.html#use_parameters_of',
                                                                                              'fastkafka/_components/meta.py')},
            'fastkafka._components.producer_decorator': { 'fastkafka._components.producer_decorator.KafkaEvent': ( 'producerdecorator.html#kafkaevent',
                                                                                                                   'fastkafka/_components/producer_decorator.py'),
                                                          'fastkafka._components.producer_decorator._to_json_utf8': ( 'producerdecorator.html#_to_json_utf8',
                                                                                                                      'fastkafka/_components/producer_decorator.py'),
                                                          'fastkafka._components.producer_decorator._wrap_in_event': ( 'producerdecorator.html#_wrap_in_event',
                                                                                                                       'fastkafka/_components/producer_decorator.py'),
                                                          'fastkafka._components.producer_decorator.producer_decorator': ( 'producerdecorator.html#producer_decorator',
                                                                                                                           'fastkafka/_components/producer_decorator.py')},
            'fastkafka._components.test_dependencies': { 'fastkafka._components.test_dependencies._install_java': ( 'test_dependencies.html#_install_java',
                                                                                                                    'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies._install_kafka': ( 'test_dependencies.html#_install_kafka',
                                                                                                                     'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies._install_testing_deps': ( 'test_dependencies.html#_install_testing_deps',
                                                                                                                            'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies.check_java': ( 'test_dependencies.html#check_java',
                                                                                                                 'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies.check_kafka': ( 'test_dependencies.html#check_kafka',
                                                                                                                  'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies.generate_app_in_tmp': ( 'test_dependencies.html#generate_app_in_tmp',
                                                                                                                          'fastkafka/_components/test_dependencies.py'),
                                                         'fastkafka._components.test_dependencies.generate_app_src': ( 'test_dependencies.html#generate_app_src',
                                                                                                                       'fastkafka/_components/test_dependencies.py')},
            'fastkafka._testing.apache_kafka_broker': { 'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker': ( 'apachekafkabroker.html#apachekafkabroker',
                                                                                                                      'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.__aenter__': ( 'apachekafkabroker.html#apachekafkabroker.__aenter__',
                                                                                                                                 'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.__aexit__': ( 'apachekafkabroker.html#apachekafkabroker.__aexit__',
                                                                                                                                'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.__enter__': ( 'apachekafkabroker.html#apachekafkabroker.__enter__',
                                                                                                                                'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.__exit__': ( 'apachekafkabroker.html#apachekafkabroker.__exit__',
                                                                                                                               'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.__init__': ( 'apachekafkabroker.html#apachekafkabroker.__init__',
                                                                                                                               'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._check_deps': ( 'apachekafkabroker.html#apachekafkabroker._check_deps',
                                                                                                                                  'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._create_topics': ( 'apachekafkabroker.html#apachekafkabroker._create_topics',
                                                                                                                                     'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._start': ( 'apachekafkabroker.html#apachekafkabroker._start',
                                                                                                                             'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._start_kafka': ( 'apachekafkabroker.html#apachekafkabroker._start_kafka',
                                                                                                                                   'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._start_service': ( 'apachekafkabroker.html#apachekafkabroker._start_service',
                                                                                                                                     'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._start_zookeeper': ( 'apachekafkabroker.html#apachekafkabroker._start_zookeeper',
                                                                                                                                       'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker._stop': ( 'apachekafkabroker.html#apachekafkabroker._stop',
                                                                                                                            'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.get_service_config_string': ( 'apachekafkabroker.html#apachekafkabroker.get_service_config_string',
                                                                                                                                                'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.is_started': ( 'apachekafkabroker.html#apachekafkabroker.is_started',
                                                                                                                                 'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.start': ( 'apachekafkabroker.html#apachekafkabroker.start',
                                                                                                                            'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.ApacheKafkaBroker.stop': ( 'apachekafkabroker.html#apachekafkabroker.stop',
                                                                                                                           'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.get_free_port': ( 'apachekafkabroker.html#get_free_port',
                                                                                                                  'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.get_kafka_config_string': ( 'apachekafkabroker.html#get_kafka_config_string',
                                                                                                                            'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.get_zookeeper_config_string': ( 'apachekafkabroker.html#get_zookeeper_config_string',
                                                                                                                                'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.run_and_match': ( 'apachekafkabroker.html#run_and_match',
                                                                                                                  'fastkafka/_testing/apache_kafka_broker.py'),
                                                        'fastkafka._testing.apache_kafka_broker.write_config_and_run': ( 'apachekafkabroker.html#write_config_and_run',
                                                                                                                         'fastkafka/_testing/apache_kafka_broker.py')},
            'fastkafka._testing.in_memory_broker': { 'fastkafka._testing.in_memory_broker.GroupMetadata': ( 'inmemorybroker.html#groupmetadata',
                                                                                                            'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.__init__': ( 'inmemorybroker.html#groupmetadata.__init__',
                                                                                                                     'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.assign_partitions': ( 'inmemorybroker.html#groupmetadata.assign_partitions',
                                                                                                                              'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.get_partitions': ( 'inmemorybroker.html#groupmetadata.get_partitions',
                                                                                                                           'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.rebalance': ( 'inmemorybroker.html#groupmetadata.rebalance',
                                                                                                                      'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.set_offset': ( 'inmemorybroker.html#groupmetadata.set_offset',
                                                                                                                       'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.subscribe': ( 'inmemorybroker.html#groupmetadata.subscribe',
                                                                                                                      'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.GroupMetadata.unsubscribe': ( 'inmemorybroker.html#groupmetadata.unsubscribe',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker': ( 'inmemorybroker.html#inmemorybroker',
                                                                                                             'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.__init__': ( 'inmemorybroker.html#inmemorybroker.__init__',
                                                                                                                      'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker._start': ( 'inmemorybroker.html#inmemorybroker._start',
                                                                                                                    'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker._stop': ( 'inmemorybroker.html#inmemorybroker._stop',
                                                                                                                   'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.connect': ( 'inmemorybroker.html#inmemorybroker.connect',
                                                                                                                     'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.dissconnect': ( 'inmemorybroker.html#inmemorybroker.dissconnect',
                                                                                                                         'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.lifecycle': ( 'inmemorybroker.html#inmemorybroker.lifecycle',
                                                                                                                       'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.read': ( 'inmemorybroker.html#inmemorybroker.read',
                                                                                                                  'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.subscribe': ( 'inmemorybroker.html#inmemorybroker.subscribe',
                                                                                                                       'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.unsubscribe': ( 'inmemorybroker.html#inmemorybroker.unsubscribe',
                                                                                                                         'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryBroker.write': ( 'inmemorybroker.html#inmemorybroker.write',
                                                                                                                   'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer': ( 'inmemorybroker.html#inmemoryconsumer',
                                                                                                               'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.__call__': ( 'inmemorybroker.html#inmemoryconsumer.__call__',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.__init__': ( 'inmemorybroker.html#inmemoryconsumer.__init__',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.getmany': ( 'inmemorybroker.html#inmemoryconsumer.getmany',
                                                                                                                       'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.start': ( 'inmemorybroker.html#inmemoryconsumer.start',
                                                                                                                     'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.stop': ( 'inmemorybroker.html#inmemoryconsumer.stop',
                                                                                                                    'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryConsumer.subscribe': ( 'inmemorybroker.html#inmemoryconsumer.subscribe',
                                                                                                                         'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer': ( 'inmemorybroker.html#inmemoryproducer',
                                                                                                               'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer.__call__': ( 'inmemorybroker.html#inmemoryproducer.__call__',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer.__init__': ( 'inmemorybroker.html#inmemoryproducer.__init__',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer.send': ( 'inmemorybroker.html#inmemoryproducer.send',
                                                                                                                    'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer.start': ( 'inmemorybroker.html#inmemoryproducer.start',
                                                                                                                     'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.InMemoryProducer.stop': ( 'inmemorybroker.html#inmemoryproducer.stop',
                                                                                                                    'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaPartition': ( 'inmemorybroker.html#kafkapartition',
                                                                                                             'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaPartition.__init__': ( 'inmemorybroker.html#kafkapartition.__init__',
                                                                                                                      'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaPartition.latest_offset': ( 'inmemorybroker.html#kafkapartition.latest_offset',
                                                                                                                           'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaPartition.read': ( 'inmemorybroker.html#kafkapartition.read',
                                                                                                                  'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaPartition.write': ( 'inmemorybroker.html#kafkapartition.write',
                                                                                                                   'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaRecord': ( 'inmemorybroker.html#kafkarecord',
                                                                                                          'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic': ( 'inmemorybroker.html#kafkatopic',
                                                                                                         'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.__init__': ( 'inmemorybroker.html#kafkatopic.__init__',
                                                                                                                  'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.latest_offset': ( 'inmemorybroker.html#kafkatopic.latest_offset',
                                                                                                                       'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.read': ( 'inmemorybroker.html#kafkatopic.read',
                                                                                                              'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.write': ( 'inmemorybroker.html#kafkatopic.write',
                                                                                                               'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.write_with_key': ( 'inmemorybroker.html#kafkatopic.write_with_key',
                                                                                                                        'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.KafkaTopic.write_with_partition': ( 'inmemorybroker.html#kafkatopic.write_with_partition',
                                                                                                                              'fastkafka/_testing/in_memory_broker.py'),
                                                     'fastkafka._testing.in_memory_broker.split_list': ( 'inmemorybroker.html#split_list',
                                                                                                         'fastkafka/_testing/in_memory_broker.py')},
            'fastkafka._testing.local_redpanda_broker': { 'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker': ( 'localredpandabroker.html#localredpandabroker',
                                                                                                                            'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.__aenter__': ( 'localredpandabroker.html#localredpandabroker.__aenter__',
                                                                                                                                       'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.__aexit__': ( 'localredpandabroker.html#localredpandabroker.__aexit__',
                                                                                                                                      'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.__enter__': ( 'localredpandabroker.html#localredpandabroker.__enter__',
                                                                                                                                      'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.__exit__': ( 'localredpandabroker.html#localredpandabroker.__exit__',
                                                                                                                                     'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.__init__': ( 'localredpandabroker.html#localredpandabroker.__init__',
                                                                                                                                     'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker._check_deps': ( 'localredpandabroker.html#localredpandabroker._check_deps',
                                                                                                                                        'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker._create_topics': ( 'localredpandabroker.html#localredpandabroker._create_topics',
                                                                                                                                           'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker._start': ( 'localredpandabroker.html#localredpandabroker._start',
                                                                                                                                   'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker._start_redpanda': ( 'localredpandabroker.html#localredpandabroker._start_redpanda',
                                                                                                                                            'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker._stop': ( 'localredpandabroker.html#localredpandabroker._stop',
                                                                                                                                  'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.get_service_config_string': ( 'localredpandabroker.html#localredpandabroker.get_service_config_string',
                                                                                                                                                      'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.is_started': ( 'localredpandabroker.html#localredpandabroker.is_started',
                                                                                                                                       'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.start': ( 'localredpandabroker.html#localredpandabroker.start',
                                                                                                                                  'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.LocalRedpandaBroker.stop': ( 'localredpandabroker.html#localredpandabroker.stop',
                                                                                                                                 'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.check_docker': ( 'localredpandabroker.html#check_docker',
                                                                                                                     'fastkafka/_testing/local_redpanda_broker.py'),
                                                          'fastkafka._testing.local_redpanda_broker.get_redpanda_docker_cmd': ( 'localredpandabroker.html#get_redpanda_docker_cmd',
                                                                                                                                'fastkafka/_testing/local_redpanda_broker.py')},
            'fastkafka._testing.test_utils': { 'fastkafka._testing.test_utils.display_docs': ( 'test_utils.html#display_docs',
                                                                                               'fastkafka/_testing/test_utils.py'),
                                               'fastkafka._testing.test_utils.mock_AIOKafkaProducer_send': ( 'test_utils.html#mock_aiokafkaproducer_send',
                                                                                                             'fastkafka/_testing/test_utils.py'),
                                               'fastkafka._testing.test_utils.nb_safe_seed': ( 'test_utils.html#nb_safe_seed',
                                                                                               'fastkafka/_testing/test_utils.py'),
                                               'fastkafka._testing.test_utils.run_script_and_cancel': ( 'test_utils.html#run_script_and_cancel',
                                                                                                        'fastkafka/_testing/test_utils.py'),
                                               'fastkafka._testing.test_utils.true_after': ( 'test_utils.html#true_after',
                                                                                             'fastkafka/_testing/test_utils.py')},
            'fastkafka.testing': {'fastkafka.testing.dummy': ('testing_export.html#dummy', 'fastkafka/testing.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/097_Docs_Dependencies.ipynb.

# %% auto 0
__all__ = ['logger', 'npm_required_major_version', 'node_version', 'node_fname', 'node_url', 'local_path', 'tgz_path',
           'node_path']

# %% ../../nbs/097_Docs_Dependencies.ipynb 2
import asyncio
import os
import shutil
import subprocess  # nosec Issue: [B404:blacklist]
import sys
import tarfile
from pathlib import Path
from tempfile import TemporaryDirectory

from .helpers import in_notebook
from .logger import get_logger

if in_notebook():
    from tqdm.notebook import tqdm
else:
    from tqdm import tqdm

# %% ../../nbs/097_Docs_Dependencies.ipynb 4
logger = get_logger(__name__)

# %% ../../nbs/097_Docs_Dependencies.ipynb 5
npm_required_major_version = 9


def _check_npm(required_major_version: int = npm_required_major_version) -> None:
    if shutil.which("npm") is not None:
        cmd = "npm --version"
        proc = subprocess.run(  # nosec [B602:subprocess_popen_with_shell_equals_true]
            cmd,
            shell=True,
            check=True,
            capture_output=True,
        )
        major_version = int(proc.stdout.decode("UTF-8").split(".")[0])
        if major_version < required_major_version:
            raise RuntimeError(
                f"Found installed npm major version: {major_version}, required npx major version: {required_major_version}. To use documentation features of FastKafka, please update npm"
            )
    else:
        raise RuntimeError(
            f"npm not found, to use documentation generation features of FastKafka, you must have npm >= {required_major_version} installed"
        )

# %% ../../nbs/097_Docs_Dependencies.ipynb 10
node_version = "v18.15.0"
node_fname = f"node-{node_version}-linux-x64"
node_url = f"https://nodejs.org/dist/{node_version}/{node_fname}.tar.xz"
local_path = Path(os.environ["HOME"]) / ".local"
tgz_path = local_path / f"{node_fname}.tar.xz"
node_path = local_path / f"{node_fname}"


def _check_npm_with_local(node_path: Path = node_path) -> None:
    try:
        _check_npm()
    except RuntimeError as e:
        if (node_path / "bin").exists():
            logger.info("Found local installation of NodeJS.")
            os.environ["PATH"] = os.environ["PATH"] + f":{node_path}/bin"
            _check_npm()
        else:
            raise e

# %% ../../nbs/097_Docs_Dependencies.ipynb 13
def _install_node(
    *,
    node_url: str = node_url,
    local_path: Path = local_path,
    tgz_path: Path = tgz_path,
) -> None:
    try:
        import requests
    except Exception as e:
        msg = "Please install docs version of fastkafka using 'pip install fastkafka[docs]' command"
        logger.error(msg)
        raise RuntimeError(msg)

    logger.info("Installing NodeJS...")
    local_path.mkdir(exist_ok=True, parents=True)
    response = requests.get(
        node_url,
        stream=True,
    )
    try:
        total = response.raw.length_remaining // 128
    except Exception:
        total = None

    with open(tgz_path, "wb") as f:
        for data in tqdm(response.iter_content(chunk_size=128), total=total):
            f.write(data)

    with tarfile.open(tgz_path) as tar:
        for tarinfo in tar:
            tar.extract(tarinfo, local_path)

    os.environ["PATH"] = os.environ["PATH"] + f":{node_path}/bin"
    logger.info(f"Node installed in {node_path}.")

# %% ../../nbs/097_Docs_Dependencies.ipynb 16
async def _install_docs_npm_deps() -> None:
    with TemporaryDirectory() as d:
        cmd = (
            "npx -y -p @asyncapi/generator ag https://raw.githubusercontent.com/asyncapi/asyncapi/master/examples/simple.yml @asyncapi/html-template -o "
            + d
        )

        proc = await asyncio.create_subprocess_shell(
            cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )
        stdout, stderr = await proc.communicate()

        if proc.returncode == 0:
            logger.info("AsyncAPI generator installed")
        else:
            logger.error("AsyncAPI generator NOT installed!")
            logger.info(
                f"stdout of '$ {cmd}'{stdout.decode('UTF-8')} \n return_code={proc.returncode}"
            )
            logger.info(
                f"stderr of '$ {cmd}'{stderr.decode('UTF-8')} \n return_code={proc.returncode}"
            )
            raise ValueError(
                f"""AsyncAPI generator NOT installed, used '$ {cmd}'
----------------------------------------
stdout:
{stdout.decode("UTF-8")}
----------------------------------------
stderr:
{stderr.decode("UTF-8")}
----------------------------------------
return_code={proc.returncode}"""
            )
